package com.hamaf.hikyaku.access;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.RetrieveKeysInterface;
import com.hamaf.hikyaku.dataobject.utils.Dto2SqlUtils;
import com.hamaf.hikyaku.dataobject.utils.ResultSetUtil;


/**
 * <p>^CgFDefaultTableAccess is Default Table Access Class.</p>
 * <p>FWe[uANZXNX</p>
 * <br>@\
 * <ol>
 *   <li>e[uݒ肷 - setTableName(String)
 *   <li>RlNV擾p[^ݒ肷 - setConnParm(String)
 *   <li>}@\ - insert(DetailItemsInterface)
 *   <li>XV@\ - update(DetailItemsInterface)
 *   <li>폜@\ - delete(DetailItemsInterface)
 *   <li>擾@\ - findByPrimaryKey(DetailItemsInterface)
 *   <li>@\ - find(RetrieveKeysInterface)
 * </ol>
 *
 * <p>쌠: Copyright (c) 2004/11/26</p>
 * @author Hamanaka
 * @version 1.0
 */
public class DefaultTableAccess extends AccessBase {

  protected String tableName = null;
  protected String connectionParameter = null;

  private boolean _debug = false;
  private static final String CLNM = "*** DefaultTableAccess : ";

  private static final String SEL_SQL = "SELECT {0} FROM {1} {2}";
  private static final String UPD_SQL = "UPDATE {0} {1} {2}";
  private static final String IST_SQL = "INSERT INTO {0} ({1}) VALUES({2})";
  private static final String DEL_SQL = "DELETE FROM {0} {1}";

  /**
   *  Creates a new <code>DefaultTableAccess.java</code> instance.
   * <br>Default Constructor
   */
  public DefaultTableAccess() {
  }

  /**
   * Describe <code>setTableName</code> method : e[uݒ肷.
   * @param tblName a <code>String</code> : e[u
   */
  public final void setTableName(String tblName) {
    tableName = tblName;
  }

  /**
   * Describe <code>setConnectionParameter</code> method : RlNV擾p[^ݒ肷.
   * @param connPrm a <code>String</code> : RlNV擾p[^
   */
  public final void setConnectionParameter(String connPrm) {
    connectionParameter = connPrm;
  }

  /**
   * Describe <code>findByPrimaryKey</code> method : get record by primaryKey.
   * <br>w肳ꂽvC}L[l̃R[hf[^擾B
   * <br>f[^IuWFNg̎gpɂĔrp̏
   * <code>(getSelectStringWithExclusion,setExclusion)</code>𗘗pB
   * @param pdi a <code>DetailItemsInterface</code>
   * @return a <code>DetailItemsInterface</code>
   * @throws SQLException
   */
  public DetailItemsInterface findByPrimaryKey(DetailItemsInterface dii) throws SQLException {
    super.setConnection(connectionParameter, dii.getSysId());
    Statement stm = super.getStatement();
    Object[] prm = {Dto2SqlUtils.getSelectItemsWithExclusion(dii), tableName,
        Dto2SqlUtils.getWhereString(dii, getPkeys(tableName))};
    String sqlStm = MessageFormat.format(SEL_SQL, prm);
    debugOut(" (findByPrimaryKey) ", sqlStm);

    try {
      ResultSet rs = stm.executeQuery(sqlStm);
      dii.setNotFound();
      if (rs.next()) {
        dii.clearNotFound();
        ResultSetUtil.setExclusion(rs, dii);
        ResultSetUtil.setValues(rs, dii);
      }
      rs.close();
    }
    finally {
      super.destroy();
    }

    return dii;
  }

  /**
   * Describe <code>regist</code> method : insert TABLE.
   * <br>f[^o^B
   * <br>f[^IuWFNg̎gpɂĔrp̏
   * <code>(getInsertItemWithExclusion,getInsertValueWithExclusion)</code>𗘗pB
   * @param dii a <code>DetailItemsInterface</code>
   * @throws SQLException
   */
  public void regist(DetailItemsInterface dii) throws SQLException {
    super.setConnection(connectionParameter, dii.getSysId());
    Statement stm = super.getStatement();

    Object[] prm = {tableName, Dto2SqlUtils.getInsertItemWithExclusion(dii),
        Dto2SqlUtils.getInsertValueWithExclusion(dii)};
    String sqlStm = MessageFormat.format(IST_SQL, prm);
    debugOut(" (regist) ", sqlStm);

    try {
      stm.executeUpdate(sqlStm);
      _conn.commit();
    } finally {
      super.destroy();
    }
  }

  /**
   * Describe <code>update</code> method : update TABLE items by PrimaryKey.
   * <br>vC}L[TABLEXVB
   * <br>XVOɔr`FbNsB
   * <br>f[^IuWFNg̎gpɂĔrp̏
   * <code>(getUpdateStringWithExclusion)</code>𗘗pB
   *
   * @param dii a <code>DetailItemsInterface</code>
   * @throws ExclusionException : rG[̏ꍇ
   * @throws SQLException
   */
  public void update(DetailItemsInterface dii) throws ExclusionException, SQLException {
    updateExec(dii, true);
  }

  /**
   * Describe <code>updateNoExclusion</code> method : update TABLE items by PrimaryKey.
   * <br>r`FbNsȂ킸ɁAvC}L[TABLEXVB
   *
   * @param dii a <code>DetailItemsInterface</code>
   * @throws SQLException
   */
  public void updateNoExclusion(DetailItemsInterface dii) throws SQLException {
    try {
      updateExec(dii, false);
    } catch (ExclusionException ee) { }
  }
  private void updateExec(DetailItemsInterface dii, boolean excl) throws ExclusionException, SQLException {
    super.setConnection(connectionParameter, dii.getSysId());
    Statement stm = super.getStatement();

    try {
      if (excl) {
        super.exclusion(dii, tableName);
      }

      Object[] prm = {tableName, Dto2SqlUtils.getUpdateItemsWithExclusion(dii),
          Dto2SqlUtils.getWhereString(dii, getPkeys(tableName))};
      String sqlStm = MessageFormat.format(UPD_SQL, prm);
      debugOut(" (update) ", sqlStm);

      stm.executeUpdate(sqlStm);
      _conn.commit();
    } finally {
      super.destroy();
    }
  }

  /**
   * Describe <code>delete</code> method : delete TABLE items by PrimaryKey.
   * <br>vC}L[TABLE폜B
   * <br>폜Oɔr`FbNsB
   *
   * @param dii a <code>DetailItemsInterface</code>
   * @throws ExclusionException : rG[̏ꍇ
   * @throws SQLException
   */
  public void delete(DetailItemsInterface dii) throws ExclusionException, SQLException {
    deleteExec(dii, true);
  }

  /**
   * Describe <code>deleteNoExclusion</code> method : delete TABLE items by PrimaryKey.
   * <br>r`FbNsȂ킸ɁAvC}L[TABLE폜B
   *
   * @param dii a <code>DetailItemsInterface</code>
   * @throws SQLException
   */
  public void deleteNoExclusion(DetailItemsInterface dii) throws SQLException {
    try {
      deleteExec(dii, false);
    } catch (ExclusionException ee) { }
  }
  private void deleteExec(DetailItemsInterface dii, boolean excl) throws ExclusionException, SQLException {
    super.setConnection(connectionParameter, dii.getSysId());
    Statement stm = super.getStatement();

    try {
      if (excl) {
        super.exclusion(dii, tableName);
      }

      Object[] prm = {tableName, Dto2SqlUtils.getWhereString(dii, getPkeys(tableName))};
      String sqlStm = MessageFormat.format(DEL_SQL, prm);
      debugOut(" (delete) ", sqlStm);

      stm.executeUpdate(sqlStm);
      _conn.commit();
    } finally {
      super.destroy();
    }
  }

  /**
   * Describe <code>find</code> method : get primaryKey List.
   * <br>ɂăvC}[L[ݒ肵Ė߂B
   * <br>L[ڂ<code>List</code>ɐݒ肳܂B
   *
   * @param rki a <code>RetrieveKeysInterface</code>
   * @return a <code>RetrieveKeysInterface</code>
   * @throws SQLException
   */
  public RetrieveKeysInterface find(RetrieveKeysInterface rki) throws SQLException{
    super.setConnection(connectionParameter, rki.getSysId());
    Object[] prm = {Dto2SqlUtils.getSelectItems(rki, getPkeys(tableName)),
        tableName, Dto2SqlUtils.getWhereString(rki)};
    String sqlStm = MessageFormat.format(SEL_SQL, prm);
    debugOut(" (find) ", sqlStm);

    try{
      Statement stm = super.getStatement();
      ResultSet rs = stm.executeQuery(sqlStm);
      ResultSetMetaData rsm = rs.getMetaData();
      List<String> fld = new ArrayList<String>();
      for (int i = 1; i <= rsm.getColumnCount(); i++) {
        fld.add(rsm.getColumnName(i).toUpperCase());
      }
      rki.setPkeyFld(fld);
      while (rs.next()) {
        List<String> key = new ArrayList<String>();
        for (int i = 1; i <= rsm.getColumnCount(); i++) {
          key.add(ResultSetUtil.getString(rs, i));
        }
        rki.addKey(key);
      }
    } finally {
      super.destroy();
    }
    return rki;
  }

  /**
   * @param argDebug debug ݒ肵܂B
   */
  public final void setDebug() {
    _debug = true;
  }

  /**
   * Describe <code>debugOut</code> method : print sql string.
   * @param methodName
   * @param msg a <code>CharSequence</code> : message
   */
  private void debugOut(String methodName, CharSequence msg) {
    if (!_debug) {
      return;
    }
    System.out.print(CLNM);
    System.out.print(methodName);
    System.out.println(msg.toString());
  }
}
