package com.hamaf.hikyaku.access;

import java.io.IOException;
import java.io.InputStream;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import com.hamaf.hikyaku.dataobject.PreparedItems;
import com.hamaf.hikyaku.dataobject.utils.PreparedSqlUtil;
import com.hamaf.hikyaku.dataobject.utils.ResultSetUtil;


/**
 * <p>^CgFPreparedSqlAccess is Table Access with PreparedSql.</p>
 * <p>FPreparedSQLŋLqꂽSQL̃t@CŃe[uANZXsȂ</p>
 * <br>@\
 * <ol>
 *   <li>find : PreparedSQLŃe[uf[^擾
 *   <li>update : PreparedSQLŃe[uXV
 * </ol>
 * <br>p@ :
 * <pre>
 *   class XxxAccess extends PreparedSqlAccess {
 *     // nSQL̏ꍇ
 *     public PreparedItems findAbc(PreparedItems pi) throws IOException,SQLException {
 *       // XxxAccess_findAbc.sqlɋLqĂSQLs
 *       return super.find(pi);
 *     }
 *     // XVnSQL̏ꍇ
 *     public void updateXyz(PreparedItems pi) throws IOException,SQLException {
 *       // XxxAccess_updateXyz.sqlɋLqĂSQLs
 *       super.update(pi);
 *     }
 *   }
 * </pre>
 *
 * <p>쌠: Copyright (c) 2008/06/17</p>
 * @author Hamanaka
 * @version 1.0
 */
public abstract class PreparedSqlAccess extends AccessBase {

  protected String connectionParameter = null;
  protected boolean _debug = false;

  /**
   * Describe <code>find</code> method : .
   * <br>QƗp(SELECT) Prepared SQL̎sB
   * <br>class FileƓtH_[ɂSQLR}ht@C̓es
   *
   * @param pi a <code>PreparedItems</code> :
   * @return a <code>PreparedItems</code> :
   * @throws IOException
   * @throws SQLException
   */
  protected final PreparedItems find(PreparedItems pi) throws IOException,SQLException {
    InputStream is = PreparedSqlUtil.getResource();
    return find(pi, is);
  }
  /**
   * Describe <code>find</code> method : .
   * <br>QƗp(SELECT) Prepared SQL̎sB
   * <br>pXŎw肳ꂽtH_[ɂSQLR}ht@C̓es
   *
   * @param pi a <code>PreparedItems</code> :
   * @param path a <code>String</code> : SQL̃t@CtH_[
   * @return a <code>PreparedItems</code> :
   * @throws IOException
   * @throws SQLException
   */
  protected final PreparedItems find(PreparedItems pi, String path) throws IOException, SQLException {
    InputStream is = PreparedSqlUtil.getResource(path);
    return find(pi, is);
  }
  private PreparedItems find(PreparedItems pi, InputStream is)
    throws IOException, SQLException {
    String abssql = PreparedSqlUtil.getSql(is, pi.getDelLine());
    debugOut(abssql, pi.getParam());

    List<String> kl = PreparedSqlUtil.getKeyName(abssql);
    String sql = PreparedSqlUtil.cnvINParm(abssql);

    super.setConnection(connectionParameter, pi.getSysId());
    PreparedStatement ps = _conn.prepareStatement(sql);
    ResultSetMetaData rsm = ps.getMetaData();
    for (int i = 1; i <= rsm.getColumnCount(); i++) {
      pi.addRetFld(rsm.getColumnName(i).toUpperCase());
    }
    try {
      for (Map<String, ?> elm : pi.getParam()) {
        ps = setIN(ps, kl, elm);
        ResultSet rs = ps.executeQuery();
        while (rs.next()) {
          Map<String, String> ret = new HashMap<String, String>();
          for (String fld : pi.getRetFld()) {
            ret.put(fld, ResultSetUtil.getString(rs, fld));
          }
          pi.addRetVal(ret);
        }
      }
    } finally {
      super.destroy();
    }
    return pi;
  }

  /**
   * Describe <code>update</code> method : .
   * <br>XVp(INSERT, UPDATE, DELETE) Prepared SQL̎sB
   * <br>class FileƓtH_[ɂSQLR}ht@C̓es
   *
   * @param pi a <code>PreparedItems</code> :
   * @throws IOException
   * @throws SQLException
   */
  protected final void update(PreparedItems pi) throws IOException, SQLException {
    InputStream is = PreparedSqlUtil.getResource();
    update(pi, is);
  }
  /**
   * Describe <code>update</code> method : .
   * <br>XVp(INSERT, UPDATE, DELETE) Prepared SQL̎sB
   * <br>pXŎw肳ꂽtH_[ɂSQLR}ht@C̓es
   *
   * @param pi a <code>PreparedItems</code> :
   * @param path a <code>String</code> : SQL̃t@CtH_[
   * @throws IOException
   * @throws SQLException
   */
  protected final void update(PreparedItems pi, String path) throws IOException, SQLException {
    InputStream is = PreparedSqlUtil.getResource(path);
    update(pi, is);
  }
  private void update(PreparedItems pi, InputStream is)
    throws IOException, SQLException {
    String abssql = PreparedSqlUtil.getSql(is, pi.getDelLine());
    debugOut(abssql, pi.getParam());

    List<String> kl = PreparedSqlUtil.getKeyName(abssql);
    String sql = PreparedSqlUtil.cnvINParm(abssql);

    super.setConnection(connectionParameter, pi.getSysId());
    PreparedStatement ps = _conn.prepareStatement(sql);
    try {
      for (Map<String, ?> elm : pi.getParam()) {
        ps = setIN(ps, kl, elm);
        ps.executeUpdate();
      }
      _conn.commit();
    } catch (SQLException xe) {
      _conn.rollback();
      throw xe;
    } finally {
      super.destroy();
    }
  }

  private PreparedStatement setIN(PreparedStatement pi, List<String> kys, Map<String, ?> vals)
    throws SQLException {
    int p = 0;
    for (String elm : kys) {
      p++;
      pi.setObject(p, vals.get(elm));
    }
    return pi;
  }

  private void debugOut(CharSequence msg, List<Map<String, ?>> vals) {
    if (!_debug) {
      return;
    }
    System.out.print("*** ");
    System.out.print(this.getClass().getSimpleName());
    System.out.print(" : ");
    System.out.println(msg.toString());
    System.out.println(vals);
  }

  /**
   * @param debug debug ݒ肵܂B
   */
  public final void setDebug() {
    _debug = true;
  }
  /**
   * @param connectionParameter connectionParameter ݒ肵܂B
   */
  public final void setConnectionParameter(String connectionParameter) {
    this.connectionParameter = connectionParameter;
  }
}
