package com.hamaf.hikyaku.extension;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import static com.hamaf.hikyaku.distribute.ControlXml.ROOTCONT;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import com.hamaf.hikyaku.distribute.ControlXml;
import com.hamaf.hikyaku.util.XPathUtil;
import com.hamaf.hikyaku.util.XmlUtil;

/**
 * <p>^CgFMasterXmlCntlImpl is Maste Data Xml File Controler.</p>
 * <p>F}X^pXmlFilȇsB</p>
 * <br><b>MasterXmlControl</b>C^[tF[X̎NXłB
 * 
 * <p>쌠: Copyright (c) 2004/08/24</p>
 * @author y-hamanaka
 * @version 1.0
 */
public class MasterXmlCntlImpl implements MasterXmlControl {

  private static final String XPTCD = "{0}s/{0}/{0}Code";
  private static final String XPTNM = "{0}s/{0}[{0}Code=''{1}'']/{0}Name";
  private static final String XPTAL = "{0}s/{0}[{0}Code=''{1}'']";

  private Document _mstXml = null;
  private Element _mstElm = null;

  /**
   *  Creates a new <code>MasterXmlCntlImpl.java</code> instance.
   */
  public MasterXmlCntlImpl(String sysId) {
    init(sysId);
  }

  /* (non-Javadoc)
   * @see com.hamaf.hikyaku.extension.MasterXmlControl#getConstant(java.lang.String)
   */
  public final String getConstant(String constTag)
    throws IllegalStateException, IllegalArgumentException {
    return XPathUtil.getValueWithPath(_mstElm, constTag);
  }

  /* (non-Javadoc)
   * @see com.hamaf.hikyaku.extension.MasterXmlControl#getItemsList(java.lang.String)
   */
  public final List<String[]> getItemsList(String item)
    throws IllegalStateException, IllegalArgumentException {
    docCheck();
    
    List<String[]> ret = new ArrayList<String[]>();
    NodeList ndcds = getCdNodes(item);
    for (int i = 0; i < ndcds.getLength(); i++) {
      Node cdnd = ndcds.item(i);
      String[] val = new String[2];
      val[0] = cdnd.getTextContent();
      val[1] = getNameText(item, val[0]);
      ret.add(val);
    }
    return ret;
  }
  private NodeList getCdNodes(String item) throws IllegalArgumentException {
    Object[] prm = {item};
    XPath xp = XPathFactory.newInstance().newXPath();
    try {
      return (NodeList)xp.evaluate(MessageFormat.format(XPTCD, prm),
          _mstElm, XPathConstants.NODESET);
    } catch (XPathExpressionException xee) {
      throw new IllegalArgumentException(xee);
    }
  }
  private String getNameText(String item, String cd) {
    Object[] prm = {item, cd};
    XPath xp = XPathFactory.newInstance().newXPath();
    try {
      return xp.evaluate(MessageFormat.format(XPTNM, prm), _mstElm);
    } catch (XPathExpressionException xee) {
      xee.printStackTrace();
      return "";
    }
  }

  /* (non-Javadoc)
   * @see com.hamaf.hikyaku.extension.MasterXmlControl#getItemValue(java.lang.String, java.lang.String)
   */
  public final Map<String, String> getItemValue(String item, String code)
    throws IllegalStateException, IllegalArgumentException {
    docCheck();
    Object[] prm = {item, code};
    XPath xp = XPathFactory.newInstance().newXPath();
    try {
      Node elm = (Node)xp.evaluate(MessageFormat.format(XPTAL, prm),
          _mstElm, XPathConstants.NODE);
      return setItemsValue((Element)elm);
    } catch (XPathExpressionException xee) {
      throw new IllegalArgumentException(xee);
    }
  }

  /* (Javadoc Ȃ)
   * @see com.hamaf.hikyaku.extension.MasterXmlControl#getTagValue(java.lang.String, java.lang.String, java.lang.String)
   */
  public final String getTagValue(String item, String code, String tag) {
    try {
      Map<String, String> vals = getItemValue(item, code);
      if (!vals.containsKey(tag)) return "";
      return vals.get(tag);
    } catch (Exception xe) {
      xe.printStackTrace();
    }
    return "";
  }

  //-------- private methods Describe
  /**
   * Describe <code>init</code> method : create DOM from Xml-File.
   * <br>XMLŎw肳ĂMasterXmlDOMƂĕێB
   * 
   * @param sysId a <code>String</code> : vOCID
   */
  private void init(String sysId) {
    String mstfl = ControlXml.newInstance().getRootDef(sysId, ROOTCONT.MasterXml);
    if (mstfl == null || mstfl.length() == 0) {
//      System.err.println("<" + ROOTCONT.MasterXml + "> Not Found in ControlXml.");
      return;
    } // end of if (Tag"MasterXml" is not Found)
    
    try {
      _mstXml = XmlUtil.createDocument(mstfl);
      _mstElm = _mstXml.getDocumentElement();
    } catch (Exception xe) {
      xe.printStackTrace();
    } // end of try-catch
  }

  /**
   * Describe <code>docCheck</code> method : check exist DOM.
   * <br>DOMێĂȂꍇAO𓊂܂B
   * @exception IllegalStateException if an error occurs
   */
  private void docCheck() throws IllegalStateException {
    if (_mstXml == null) {
      throw new IllegalStateException("Master Xml is Null!!");
    } // end of if (Documen Object exist check)
  }

  /**
   * Describe <code>setItemsValue</code> method : setting childes tag value.
   * <br>Gg͈ȉ̎q^Oƒl̃}bv߂B
   * @param elm an <code>Element</code> :
   * @return a <code>Map</code> :
   */
  private Map<String, String> setItemsValue(Node elm) {
    Map<String, String> im = new HashMap<String, String>();
    NodeList nds = elm.getChildNodes();
    for (int i = 0; i < nds.getLength(); i++) {
      Node nd = nds.item(i);
      if (nd.getNodeType() == Node.ELEMENT_NODE) {
        im.put(nd.getNodeName(), nd.getTextContent());
      }
    }
    return im;
  }

}
