package com.hamaf.hikyaku.security.process;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import com.hamaf.hikyaku.dataobject.PreparedItems;
import com.hamaf.hikyaku.distribute.DistributeServlet;
import com.hamaf.hikyaku.security.access.BelongAccess;
import com.hamaf.hikyaku.security.access.BelongSql;
import com.hamaf.hikyaku.security.access.GroupUserAccess;
import com.hamaf.hikyaku.security.access.GroupUserSql;
import com.hamaf.hikyaku.security.dto.BelongDto;
import com.hamaf.hikyaku.security.dto.GroupUserDto;

/**
 * <p>^CgFGroupMemberProcess is .</p>
 * <p>FO[vɑ郁o[iO[v,[U[j̏s</p>
 * <br>
 * 
 * <p>쌠: Copyright (c) 2010/05/10</p>
 * @author y-hamanaka
 * @version 1.0
 */
public class GroupMemberProcess {
  
  private BelongAccess _ba = null;
  private GroupUserAccess _gua = null;
  private BelongSql _bs = null;
  private GroupUserSql _gus = null;

  /**
   *  Creates a new <code>GroupMemberProcess.java</code> instance.
   * <br>Default Constructor
   */
  public GroupMemberProcess(BelongAccess ba, GroupUserAccess gua,
      BelongSql bs, GroupUserSql gus) {
    _ba = ba;
    _gua = gua;
    _bs = bs;
    _gus = gus;
  }
  
  /**
   * Describe <code>getBelongs</code> method : .
   * <br>z̃O[v擾B
   * <br>ez BELONG_ID, GROUP_CODE, GROUP_CODE ߂B
   * 
   * @param gid a <code>String</code> : O[vID
   * @return a <code>List&le;String[3]&gt;</code> : z̃O[ṽXg 
   * @throws Exception
   */
  public List<String[]> getBelongs(String gid) throws Exception {
    List<String[]> ret = new ArrayList<String[]>();
    PreparedItems pi = createPre(gid);
    pi = _bs.getChildren(pi);
    for (Map<String, String> dt : pi.getRetVal()) {
    	String[] dts = {dt.get("BELONG_ID"), dt.get("GROUP_CODE"),  dt.get("GROUP_CODE")};
      ret.add(dts);
    }    
    return ret;
  }
  
  /**
   * Describe <code>getUsers</code> method : .
   * <br>郆[U[擾B
   * <br>ez USER_ID, USER_NAME, ORGANIZATION ߂B
   * 
   * @param gid a <code>String</code> : O[vID
   * @return a <code>List&le;String[3]&gt;</code> : 郆[U[̃Xg 
   * @throws Exception
   */
  public List<String[]> getUsers(String gid) throws Exception {
    List<String[]> ret = new ArrayList<String[]>();
    PreparedItems pi = createPre(gid);
    pi = _gus.getUsers(pi);
    for (Map<String, String> dt : pi.getRetVal()) {
    	String[] dts = {dt.get("USER_ID"), dt.get("UNAME"),  dt.get("ORGANIZATION")};
      ret.add(dts);
    }    
    return ret;
  }
  
  private PreparedItems createPre(String gid) {
    PreparedItems pi = new PreparedItems();
    pi.setSysId(DistributeServlet.SYSID);
    pi.addDelLine("in");
    Map<String, String> vals = new HashMap<String, String>();
    vals.put("GroupId", gid);
    pi.addParam(vals);
    return pi;
  }
  
  /**
   * Describe <code>getParent</code> method : .
   * <br>eO[v擾B
   * <br>
   * @param gid a <code>String</code> : O[vID
   * @return a <code>List&lt;String&gt;</code> : O[ṽXg
   * @throws Exception
   */
  public List<String> getParent(String gid) throws Exception {
    List<String> ret = new ArrayList<String>();
    PreparedItems pi = new PreparedItems();
    pi.setSysId(DistributeServlet.SYSID);
    pi.addDelLine("in");
    Map<String, String> vals = new HashMap<String, String>();
    vals.put("BelongId", gid);
    pi.addParam(vals);
    pi = _gus.getGroups(pi);
    for (Map<String, String> dt : pi.getRetVal()) {
      ret.add(dt.get("GROUP_ID"));
    }    
    return ret;
  }
  
  /**
   * Describe <code>getGroupByUser</code> method : .
   * <br>[U[̑O[v擾B
   * <br>
   * @param uid a <code>String</code> : [U[ID
   * @return a <code>List&lt;String&gt;</code> : O[ṽXg
   * @throws Exception
   */
  public List<String> getGroupByUser(String uid) throws Exception {
    List<String> ret = new ArrayList<String>();
    PreparedItems pi = new PreparedItems();
    pi.setSysId(DistributeServlet.SYSID);
    pi.addDelLine("in");
    Map<String, String> vals = new HashMap<String, String>();
    vals.put("UserId", uid);
    pi.addParam(vals);
    pi = _gus.getGroups(pi);
    for (Map<String, String> dt : pi.getRetVal()) {
      ret.add(dt.get("GROUP_ID"));
    }    
    return ret;
  }
  
  /**
   * Describe <code>changeBelong</code> method : .
   * <br>O[vz̃O[vւB
   * 
   * @param gid a <code>String</code> : O[vID
   * @param bids a <code>List</code> : z̃O[vID
   * @throws Exception
   */
  public void changeBelong(String gid, List<String> bids) throws Exception {
    PreparedItems pi = createPre(gid);
    BelongDto bd = new BelongDto();
    bd.setItemValue(BelongDto.GROUP_ID, gid);
    
    _bs.deleteByGroupId(pi);
    for (String bid : bids) {
      bd.setItemValue(BelongDto.BELONG_ID, bid);
      _ba.regist(bd);
    }
  }

  /**
   * Describe <code>changeUser</code> method : .
   * <br>O[vɑ郆[U[ւB
   * 
   * @param gid a <code>String</code> : O[vID
   * @param users a <code>List</code> : 郆[U[ID
   * @throws Exception
   */
  public void changeUser(String gid, List<String> users) throws Exception {
    PreparedItems pi = createPre(gid);
    GroupUserDto gud = new GroupUserDto();
    gud.setItemValue(GroupUserDto.GROUP_ID, gid);
    
    _gus.deleteByGroupId(pi);
    for (String user : users) {
      gud.setItemValue(GroupUserDto.USER_ID, user);
      _gua.regist(gud);
    }
  }
  
}
