package com.hamaf.hikyaku.security.process;

import java.security.SecureRandom;
import java.util.Calendar;
import java.util.Map;
import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;
import javax.servlet.http.HttpSession;
import com.hamaf.hikyaku.distribute.DistributeServlet;
import com.hamaf.hikyaku.extension.MasterXmlControl;
import com.hamaf.hikyaku.extension.MasterXmlFactory;
import com.hamaf.hikyaku.security.dto.UserDto;
import com.hamaf.hikyaku.security.ui.LogInCheck;
import com.hamaf.hikyaku.util.DateUtil;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

/**
 * <p>^CgFPasswordUtil is .</p>
 * <p>F</p>
 * <br>
 * 
 * <p>쌠: Copyright (c) 2010/03/16</p>
 * @author y-hamanaka
 * @version 1.0
 */
public class PasswordUtil {

  public final static String MXTAG = "password";
  public final static String MXKEY = "cd";
  
  private final static String MXMIN = "minlength";
  private final static String MXMAX = "maxlength";
  private final static String MXMIX = "mixed";
  private final static String MXENC = "encryption";
  private final static String MXCKY = "cryptKey";
  private final static String MXYES = "yes";
  
  private final static String MINLEN = "8";
  private final static String ALG = "DES";
  private final static String TYP = "DES/ECB/PKCS5Padding";

  private final static String NUMCHR = "\\p{Digit}";
  private final static String ENGCHR = "\\p{Alpha}";
  private final static String SIGCHR = "\\p{Punct}";
  
  /**
   * Describe <code>checkChar</code> method : pX[h̗Lނ̃`FbN.
   * <br>8`20ŉpAAL̂1ȏgpĂ邱
   * @param pswd a <code>String</code> : pX[h
   * @return a <code>boolean</code> : G[̏ꍇA<b>true</b>
   * @exception Exception if an error occurs
   */
  public static boolean checkChar(String pswd) {
    MasterXmlControl mxc = MasterXmlFactory.newInstance().getMasterXml(DistributeServlet.SYSID);
    Map<String, String> pwdinf = mxc.getItemValue(MXTAG, MXKEY);
    StringBuffer sb = new StringBuffer();
    sb.append("[");
    sb.append(NUMCHR);
    sb.append(ENGCHR);
    sb.append(SIGCHR);
    sb.append("]{");
    String min = pwdinf.get(MXMIN);
    if (min == null || min.length() == 0) min = MINLEN;
    sb.append(min);
    sb.append(",");
    String max = pwdinf.get(MXMAX);
    if (max == null || max.length() == 0 || Integer.parseInt(max) < Integer.parseInt(min)) max = "999";
    sb.append(max);
    sb.append("}");

    if (!pswd.matches(sb.toString())) {
      return true;
    }
    
    if (!MXYES.equals(pwdinf.get(MXMIX))) return false;
    
    boolean chk = true;
    chk = chk && pswd.matches(".*[" + NUMCHR + "]+.*");
    chk = chk && pswd.matches(".*[" + ENGCHR + "]+.*");
    chk = chk && pswd.matches(".*[" + SIGCHR + "]+.*");
    return !chk;
  }
  
  /**
   * Describe <code>checkPeriod</code> method : ݓt`FbNԓ`FbN.
   * <br><b>Ԃ̎w@</b>
   * <br>format : Jn-͈  "nn[Y|M|D|W]"`Ŏw肷
     <br> FQ̂R --> 2M-3D A ŜPT --> 4M-1W
   * @param modDate a <code>String</code> : ͈̓`FbNƂȂN
   * @param period a <code>String</code> : `FbN
   * @return a <code>boolean</code> : ͈͓̏ꍇA<b>true</b>߂
   * @exception Exception if an error occurs
   */
  public static boolean checkPeriod(String modDate, String period) throws Exception {
    if (modDate == null || modDate.length() == 0) return false;
    if (period == null || period.length() == 0) return false;
    
    String[] dt = period.split("-");
    Calendar ndt = Calendar.getInstance();

    Calendar sdt = DateUtil.getCalendar(modDate);
    int val = Integer.parseInt(dt[0].substring(0, dt[0].length() - 1));
    int typ = convAmountType(dt[0].charAt(dt[0].length() - 1));
    sdt.add(typ, val);
    Calendar edt = (Calendar)sdt.clone();
    val = Integer.parseInt(dt[1].substring(0, dt[1].length() - 1));
    typ = convAmountType(dt[1].charAt(dt[1].length() - 1));
    edt.add(typ, val);
    return ndt.after(sdt) && ndt.before(edt);
  }
  private static int convAmountType(char typ) throws Exception {
    switch (typ) {
    case 'D':
    case 'd':
      return Calendar.DATE;
    case 'M':
    case 'm':
      return Calendar.MONTH;
    case 'Y':
    case 'y':
      return Calendar.YEAR;
    case 'W':
    case 'w':
      return Calendar.WEEK_OF_MONTH;
    default : throw new IllegalArgumentException("Amount Type Error : " + typ);
    }
  }
  
  /**
   * Describe <code>encrypt</code> method : encrypt password and Base64Encode.
   * <br>pX[hÍBase64encode߂B
   * @param pswd a <code>String</code> : pX[h
   * @return a <code>String</code> : ÍAGR[h
   * @exception Exception if an error occurs
   */
  public static String encrypt(String pswd) throws Exception {
    if (pswd == null || pswd.length() == 0) return pswd;
    
    MasterXmlControl mxc = MasterXmlFactory.newInstance().getMasterXml(DistributeServlet.SYSID);
    Map<String, String> pwdinf = mxc.getItemValue(MXTAG, MXKEY);
    if (!MXYES.equals(pwdinf.get(MXENC))) return pswd;    
    String cky = pwdinf.get(MXCKY);
    if (cky == null || cky.length() == 0) return pswd;
    
    SecretKeySpec sks = new SecretKeySpec(cky.getBytes(), ALG);
    Cipher cp = Cipher.getInstance(TYP);
    cp.init(Cipher.ENCRYPT_MODE, sks);
    byte[] encdt = cp.doFinal(pswd.getBytes());
    BASE64Encoder b6enc = new BASE64Encoder();
    return b6enc.encode(encdt);
  }

  /**
   * Describe <code>decrypt</code> method : Base64Decode and decrypt password.
   * <br>pX[hBase64DecordA߂B
   * @param pswd a <code>String</code> : Íꂽ
   * @return a <code>String</code> : fR[hA
   * @exception Exception if an error occurs
   */
  public static String decrypt(String pswd) throws Exception {
    if (pswd == null || pswd.length() == 0) return pswd;
    
    MasterXmlControl mxc = MasterXmlFactory.newInstance().getMasterXml(DistributeServlet.SYSID);
    Map<String, String> pwdinf = mxc.getItemValue(MXTAG, MXKEY);
    if (!MXYES.equals(pwdinf.get(MXENC))) return pswd;    
    String cky = pwdinf.get(MXCKY);
    if (cky == null || cky.length() == 0) return pswd;
    
    SecretKeySpec sks = new SecretKeySpec(cky.getBytes(), ALG);
    Cipher cp = Cipher.getInstance(TYP);
    cp.init(Cipher.DECRYPT_MODE, sks);
    BASE64Decoder b6dec = new BASE64Decoder();
    byte[] decdt = cp.doFinal(b6dec.decodeBuffer(pswd));
    return new String(decdt);
  }
  
  /**
   * Describe <code>initialPassword</code> method : return random string.
   * <br>pX[hp̃_ȉpLȂ镶𐶐Ė߂B
   * 
   * @return a <code>String</code> : 
   */
  public static String initialPassword(){
    MasterXmlControl mxc = MasterXmlFactory.newInstance().getMasterXml(DistributeServlet.SYSID);
    Map<String, String> pwdinf = mxc.getItemValue(MXTAG, MXKEY);
    String size = pwdinf.get(MXMIN);
    if (size == null || size.length() == 0) size = MINLEN;
    
    int sz = Integer.parseInt(size);
    String pwd = createString(sz);
    while(checkChar(pwd)) {
      pwd = createString(sz);
    }    
    return pwd;
  }
  private static String createString(int size) {
    SecureRandom random = new SecureRandom();
    char[] pass = new char[size];

    for(int k = 0; k < pass.length; k++){
      switch (random.nextInt(4)){
        case 0: // 'a' - 'z'
          pass[k] = (char)(97 + random.nextInt(26));
          break;
        case 1: // 'A' - 'Z'
          pass[k] = (char)(65 + random.nextInt(26));
          break;
        case 2: // '0' - '9'
          pass[k] = (char)(48 + random.nextInt(10));
          break;
        case 3: // '!' - '/'
          char pw = (char)(33 + random.nextInt(15));
          while (pw == '"' || pw == ',' || pw == '\'' || pw == '&'
              || pw == '%' || pw == '#') {
            pw = (char)(33 + random.nextInt(15));
          }
          pass[k] = pw;
          break;
        default:
          pass[k] = 'a';
      }
    }
    return new String(pass);
  }
  
  /**
   * Describe <code>getUser</code> method : get userId in session.
   * <br>ZbVɕێĂ郆[U[񂩂烆[U[ID擾B
   * 
   * @param session a <code>HttpSession</code> : 
   * @return a <code>String</code> : [U[ID
   */
  public static String getUser(HttpSession session) {
    UserDto ud = (UserDto)session.getAttribute(LogInCheck.USER_INF);
    if (ud == null) return "unnonUser";
    return ud.getItemValue(UserDto.USER_ID);
  }
}
