package com.hamaf.hikyaku.security.ui;

import java.text.MessageFormat;
import java.util.List;
import com.hamaf.hikyaku.dataobject.CommonMessage;
import com.hamaf.hikyaku.dataobject.KeyList;
import com.hamaf.hikyaku.dataobject.CommonMessage.STATUS;
import com.hamaf.hikyaku.security.dto.VGroupDto;
import com.hamaf.hikyaku.security.process.GroupMemberProcess;
import com.hamaf.hikyaku.security.process.GroupProcess;
import com.hamaf.hikyaku.ui.ProcessBase;


/**
 * <p>^CgFGroupMemberModIni.</p>
 * <p>FO[vo[\ʗpf[^ݒ</p>
 * <br>
 * 
 * <p>쌠: Copyright (c) 2010/05/23</p>
 * @author yoshinori
 * @version 1.0
 */
public class GroupMemberModIni extends ProcessBase {
	
	public static final String RM_KEY = "gid";
	public static final String RM_UID = "uids";
	public static final String RM_GID = "cgids";

	private GroupMemberProcess _gmp = null;
	private GroupProcess _gp = null;

	/**
	 *  Creates a new <code>GroupMemberModIni.java</code> instance.
	 * <br>Default Constructor
	 * @param gmp
	 */
	public GroupMemberModIni(GroupMemberProcess gmp, GroupProcess gp) {
		_gmp = gmp;
		_gp = gp;
	}

	private final String M_POS = "pos";
	private final String M_GID = "gid";
  private final String RM_GRP = "dispGroup";
	private final String RM_UDT = "uvals";
	private final String RM_GDT = "cgvals";
	/* (Javadoc Ȃ)
	 * @see com.hamaf.hikyaku.ui.ProcessBase#exec(com.hamaf.hikyaku.dataobject.CommonMessage)
	 */
	@Override
	public CommonMessage exec(CommonMessage msg) throws Exception {
		String gid = getKey4KL(msg);
		if (gid == null) gid = msg.getReqStr(M_GID);
		if (gid.length() == 0) return setErrorInf(msg);
		
		msg.setResMsg(RM_KEY, gid);
		msg.setResMsg(RM_GRP, getDispGrp(gid));
		msg = setUserData(msg, gid);
		msg = setGroupData(msg, gid);
		return msg;
	}
	
	private CommonMessage setUserData(CommonMessage msg, String gid) throws Exception {
		List<String[]> udts = _gmp.getUsers(gid);
		StringBuilder sbk = new StringBuilder();
		StringBuilder sbd = new StringBuilder();
		for (String[] dts : udts) {
			sbk.append(dts[0]);
			sbk.append(",");
			sbd.append(dts[0]);
			sbd.append("-");
			sbd.append(dts[1]);
			sbd.append(",");
		}
		if (sbk.length() > 0) {
			sbk.setLength(sbk.length() - 1);
			sbd.setLength(sbd.length() - 1);
		}
		msg.setResMsg(RM_UID, sbk.toString());
		msg.setResMsg(RM_UDT, sbd.toString());
		return msg;
	}
	
	private CommonMessage setGroupData(CommonMessage msg, String gid) throws Exception {
		List<String[]> udts = _gmp.getBelongs(gid);
		StringBuilder sbk = new StringBuilder();
		StringBuilder sbd = new StringBuilder();
		for (String[] dts : udts) {
			sbk.append(dts[0]);
			sbk.append(",");
			sbd.append(dts[1]);
			sbd.append("-");
			sbd.append(dts[2]);
			sbd.append(",");
		}
		if (sbk.length() > 0) {
			sbk.setLength(sbk.length() - 1);
			sbd.setLength(sbd.length() - 1);
		}
		msg.setResMsg(RM_GID, sbk.toString());
		msg.setResMsg(RM_GDT, sbd.toString());
		return msg;
	}

	private String getKey4KL(CommonMessage msg) {
    if (super.saveObj == null) {
      super.log.fine("primaryKey not set : saveObj is null");
      return null;
    }
    String spos = msg.getReq(M_POS);
    if (spos == null || spos.length() == 0) {
      super.log.fine("primaryKey not set : 'pos' not in request");
      return null;
    }

    KeyList kyl = (KeyList)super.saveObj;
    int ps = Integer.parseInt(spos);
    List<String> kyel = kyl.getKey(ps);
		return kyel.get(0);
	}
	
	private String getDispGrp(String gid) throws Exception {
	  VGroupDto vgd = new VGroupDto();
	  vgd.setItemValue(VGroupDto.GROUP_ID, gid);
	  vgd.setRequestItem(VGroupDto.NAME);
	  vgd = _gp.findByPrimarykey(vgd);
	  StringBuilder sb = new StringBuilder();
	  sb.append(gid);
	  sb.append(" - ");
	  sb.append(vgd.getItemValue(VGroupDto.NAME));
	  return sb.toString();
	}
	
	private final String ERRM = "GrMbErrNotKey";
	private CommonMessage setErrorInf(CommonMessage msg) {
		msg.setErrorStatus(STATUS.WARNING);
		Object[] prm = new Object[2];
		prm[0] = M_POS + ":" + msg.getReq(M_POS);
		prm[1] = M_GID + ":" + msg.getReq(M_GID);
		msg.addMessage(MessageFormat.format(resource.getString(ERRM), prm));
		return msg;
	}
}
