package com.hamaf.hikyaku.security.ui;

import java.util.Map;
import com.hamaf.hikyaku.dataobject.CommonMessage;
import com.hamaf.hikyaku.dataobject.CommonMessage.STATUS;
import com.hamaf.hikyaku.distribute.DistributeServlet;
import com.hamaf.hikyaku.extension.MasterXmlControl;
import com.hamaf.hikyaku.extension.MasterXmlFactory;
import com.hamaf.hikyaku.security.access.UserAccess.USER_TYPE;
import com.hamaf.hikyaku.security.dto.UserPwdDto;
import com.hamaf.hikyaku.security.process.PasswordUtil;
import com.hamaf.hikyaku.security.process.UserProcess;
import com.hamaf.hikyaku.ui.ProcessBase;

/**
 * <p>^CgFLogInCheck.</p>
 * <p>FOCʏ</p>
 * <br>
 * 
 * <p>쌠: Copyright (c) 2010/03/31</p>
 * @author y-hamanaka
 * @version 1.0
 */
public class LogInCheck extends ProcessBase {

  public static final String USER_INF = "LogInCheck.UserInf";
  
  private UserProcess _up = null;
  
  /**
   *  Creates a new <code>LogInCheck.java</code> instance.
   * <br>Default Constructor
   */
  public LogInCheck(UserProcess up) {
    _up = up;
  }

  private final String GUEST = "guest";
  private final String UID = "loginuserid";
  private final String PWD = "loginpassword";
  private final String EMSG = "loginErrMsg";
  private final String EMSG_UNM = "loginErrMsgUnMatch";
  private final String EMSG_TERM = "loginErrMsgTerm";
  private final String EMSG_CNT = "loginErrMsgCount";
  
  /* (Javadoc Ȃ)
   * @see com.hamaf.hikyaku.ui.ProcessBase#exec(com.hamaf.hikyaku.dataobject.CommonMessage)
   */
  @Override
  public CommonMessage exec(CommonMessage msg) throws Exception {
    String uid = msg.getReqStr(UID);
    if (uid.length() == 0) uid = GUEST;
    
    UserPwdDto pwd = _up.getUserPwdInf(uid, session);
    String errMsg = null;
    if (pwd.isNotFound()) {
    	errMsg = resource.getString(EMSG_UNM);
    } else {
      errMsg = checkEnv(pwd);
      if (errMsg == null) errMsg = checkPswd(pwd, msg.getReqStr(PWD));
      if (errMsg == null) {
      	_up.setErrorCount(uid, 0, session);
      	session.setAttribute(USER_INF, _up.getDetailUserById(uid, session));
      	return msg;
      }
    }
    msg.setErrorStatus(STATUS.WARNING);
    msg.addMessage(resource.getString(EMSG));
    msg.addMessage(errMsg);
    return msg;
  }

  private final String MXTAG = "password";
  private final String MXKEY = "cd";
  private final String MXCNT = "errorCount";
  private final String MXVAL = "validity";

  private String checkEnv(UserPwdDto upd) {
    String utyp = upd.getItemValue(UserPwdDto.USER_TYPE);
    if (!USER_TYPE.GENERAL.getVal().equals(utyp)) return null;
    
    MasterXmlControl mxc = MasterXmlFactory.newInstance().getMasterXml(DistributeServlet.SYSID);
    Map<String, String> pwdinf = mxc.getItemValue(MXTAG, MXKEY);
    String strMax = pwdinf.get(MXCNT);
    if (strMax != null && strMax.length() > 0 && !"0".equals(strMax)) {
      int maxc = Integer.parseInt(strMax);
      int dcnt = Integer.parseInt(upd.getItemValue(UserPwdDto.ERROR_COUNT));
      if (dcnt > maxc) return resource.getString(EMSG_CNT);
    }
    
    String val = pwdinf.get(MXVAL);
    if (val != null && val.length() > 0 && !"0".equals(val)) {
      StringBuilder sb = new StringBuilder();
      sb.append("0d-");
      sb.append(val);
      try {
        if (!PasswordUtil.checkPeriod(upd.getItemValue(UserPwdDto.PWD_UPDATE), sb.toString())) {
          return resource.getString(EMSG_TERM);
        }
      } catch (Exception xe) {}
    }
    
    return null;
  }
  
  private final String SPUSR_PWD = "superAdminPassword";
  private String checkPswd(UserPwdDto upd, String pswd) throws Exception {
    String utyp = upd.getItemValue(UserPwdDto.USER_TYPE);
    if ("a".equals(utyp)) return null;
    
    String cps = null;
    if ("s".equals(utyp)) {
      cps = upd.getItemValue(UserPwdDto.PASSWORD);
      if (cps == null || cps.length() == 0) {
      	MasterXmlControl mxc = MasterXmlFactory.newInstance().getMasterXml(DistributeServlet.SYSID);
      	cps = mxc.getConstant(SPUSR_PWD);
      }
    } else {
      cps = PasswordUtil.decrypt(upd.getItemValue(UserPwdDto.PASSWORD));
    }
    if (!cps.equals(pswd)) return resource.getString(EMSG_UNM);
    return null;
  }
  
}
