package com.hamaf.hikyaku.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLDecoder;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import com.hamaf.hikyaku.distribute.ServletUtils;

/**
 * <p>^CgFComUtils is Common Utility Class.</p>
 * <p>Fʃ[eBeBNXłB@ȉ̋@\܂B</p>
 * <br> S <code>static</code>\bhƂĒ`Ă܂B
 * <ol>
 *   <li> ΃pX΃pXɕϊ
 *   <li> WindowsNCAg̑Mf[^̕ϊ
 *   <li> UNICODESUN --> Windows code ֕ϊ
 *   <li> UNIXnNCAg̑Mf[^ϊ
 *   <li> TagEntityQƕւ̕ϊi^O̖j
 * </ol>
 *
 * <p>쌠: Copyright (c) 2003/12/01</p>
 * @author Hamanaka
 * @version 1.0
 */
public class ComUtils {

  private static final char[] SUN = {'\u301C','\u2016','\u2212','\u00A2','\u00A3','\u00AC'};
  private static final char[] WIN = {'\uFF5E','\u2225','\uFF0D','\uFFE0','\uFFE1','\uFFE2'};
  private static final String SJIS = "Shift_JIS";

  /**
   * Describe <code>convPath</code> method : convert environment variable.
   * <br>pX"${...}"̕ϐŒuB
   *
   * @param path a <code>String</code> : ϊpX
   * @return a <code>String</code> : ϊ̃pX
   */
  public final static String convPath(String path, boolean endSep) {
    if (path == null || path.length() == 0) {
      return "";
    }
    Pattern pn = Pattern.compile("\\$\\{.+?\\}");
    Matcher mt = pn.matcher(path);
    StringBuilder sb = new StringBuilder();
    int sp = 0;
    while (mt.find()) {
      sb.append(path.substring(sp, mt.start()));
      String prm = mt.group().substring(2);
      prm = prm.substring(0, prm.length() - 1);
      String dt = null;
      if (prm.equals("APP_ROOT")) {
        dt = ServletUtils.whichFile("/");
      } else {
        dt = System.getenv(prm);
      }
      if (dt == null || dt.length() == 0) {
          dt = System.getProperty(prm);
      }
      sb.append(dt);
      sp = mt.end();
    }
    sb.append(path.substring(sp));
    if (endSep) {
      if (sb.charAt(sb.length() - 1) != File.separatorChar
          && sb.charAt(sb.length() - 1) != '/') {
        sb.append(File.separator);
      }
    }
    return sb.toString().replaceAll("\\\\/", File.separator);
  }

  /**
   * Describe <code>getResource</code> method : get resource.
   * <br>\[X܂̓t@C <code>InputStream</code> 擾B
   *
   * @param fnm a <code>String</code> : \[X
   * @return a <code>InputStream</code> : 擾InputStream
   */
  public final static InputStream getResource(String fnm) {
    String path = convPath(fnm, false);
    ClassLoader cl = Thread.currentThread().getContextClassLoader();
    InputStream is = cl.getResourceAsStream(path);
    if (is != null) return is;

    File fl = new File(path);
    if (!fl.exists()) return null;
    try {
      return new FileInputStream(fl);
    } catch (Exception xe) { }
    return null;
  }

  /**
   * Describe <code>whichFile</code> method : Change Full Path from file.
   * <br>΃pX΃pXɕϊB
   * @param fileName a <code>String</code> : ΃pXŎꂽt@C
   * @return a <code>String</code> : ΃pXɕϊ
   * @throws FileNotFoundException : t@C݂܂
   * @throws UnsupportedEncodingException : VXevpeB file.encoding 擾ł܂
   */
  public static final String whichFile(String fileName)
       throws FileNotFoundException, UnsupportedEncodingException {
    ClassLoader cl = Thread.currentThread().getContextClassLoader();
    URL url = cl.getResource(convPath(fileName, false));
    final String enc = System.getProperty("file.encoding");
    return URLDecoder.decode(url.getFile(), enc);
  }

  /**
   * Describe <code>convertSjis</code> method : Convert Windows send data to JIS.
   * <br>WindowsNCAg瑗Mꂽf[^JavapJISR[hɕϊĖ߂B
   * <br>Mf[^ɓ{͊܂܂ꍇ́Â܂܂ł͕܂B
   * @param parm a <code>String</code> : MĐf[^
   * @return a <code>String</code> : ϊ̃f[^
   * @exception UnsupportedEncodingException if an error occurs
   */
  public static final String convertSjis(String parm)
    throws UnsupportedEncodingException {
    return new String(parm.getBytes("8859_1"), "JISAutoDetect");
  }

  /**
   * Describe <code>sun2win</code> method : SUN unicode to Windows unicode.
   * <br> UNICODEϊɂMicrosoftSUÑR[ḧႢϊ܂B
   * <br> SUN --> Windows code ֕ϊB
   * @param s a <code>String</code> : ϊO̕
   * @return a <code>String</code> :@ϊ̕
   */
  public static final String sun2win(String s) {
    if (s == null) {
      return "";
    }
    StringBuffer sb = new StringBuffer(s);
    for (int l = 0; l < sb.length(); l++) {
      for (int p = 0; p < 6; p++) {
        if (sb.charAt(l) == SUN[p]) {
          sb.setCharAt(l, WIN[p]);
          break;
        }
      } // end of for (p < 6)
    } // end of for (l < sb.length())
    return sb.toString();
  }

  /**
   * Describe <code>sun2win</code> method : SUN unicode to Windows unicode.
   * <br>web.xml̕R[h(&lt;param-name&gt;CharCode)Shift_JIS̏ꍇɁA
   * <br> UNICODEϊɂMicrosoftSUÑR[ḧႢϊ܂B
   *
   * @param s a <code>String</code> : ϊO̕
   * @param charCd a <code>String</code> : characte encording
   * @return a <code>String</code> : ϊ̕
   */
  public static final String sun2win(String s, String charCd) {
    if (SJIS.equals(charCd)) {
      return sun2win(s);
    }

    return s;
  }

  /**
   * Describe <code>win2sun</code> method : Windows unicode to SUN unicode.
   * <br> UNICODEϊɂMicrosoftSUÑR[ḧႢϊ܂B
   * <br> Windows --> SUN code ֕ϊB
   * @param s a <code>String</code> : ϊO̕
   * @return a <code>String</code> :@ϊ̕
   */
  public static final String win2sun(String s) {
    if (s == null) {
      return "";
    }
    StringBuffer sb = new StringBuffer(s);
    for (int l = 0; l < sb.length(); l++) {
      for (int p = 0; p < 6; p++) {
        if (sb.charAt(l) == WIN[p]) {
          sb.setCharAt(l, SUN[p]);
          break;
        }
      } // end of for (p < 6)
    } // end of for (l < sb.length())
    return sb.toString();
  }

  /**
   * Describe <code>win2sun</code> method : Windows unicode to SUN unicode.
   * <br>web.xml̕R[h(&lt;param-name&gt;CharCode)Shift_JIS̏ꍇɁA
   * <br> UNICODEϊɂMicrosoftSUÑR[ḧႢϊ܂B
   *
   * @param s a <code>String</code> : ϊO̕
   * @param charCd a <code>String</code> : characte encording
   * @return a <code>String</code> : ϊ̕
   */
  public static final String win2sun(String s, String charCd) {
    if (SJIS.equals(charCd)) {
      return win2sun(s);
    }

    return s;
  }

  /**
   * Describe <code>filter</code> method : conver tag charcter.
   * <br> NXTCgXNveBOւ̑Ήi܂Ȃj
   * <br> TAG<code>(<,>,&)</code>𖳌ɂďʕ\̍ۂ̈Smۂ܂
   * <br>(&nbsp;)IEŃGeBeBւ̕ϊsȂߒ~B2006/12/06
   * @param str a <code>String</code> :
   * @return a <code>String</code> :
   */
  public static final String filter(String str) {
    if (str == null ) {
      return "";
    } // end of if ()

    //^OȂ΂̂܂ܕԂ
    if((str.indexOf('<') < 0) &&
       (str.indexOf('>') < 0) &&
       (str.indexOf('&') < 0) &&
//       (str.indexOf(' ') < 0) &&
       (str.indexOf('\"') < 0)) {
      return str;
    }
    //^O̗vfϊĂԂ
    StringBuffer sb = new StringBuffer();
    for(int i=0; i < str.length() ;i++){
      char c = str.charAt(i);
      if(c == '<')
        sb.append("&lt;");
      else if(c == '>')
        sb.append("&gt;");
      else if(c == '&')
        sb.append("&amp;");
      else if(c == '\"')
        sb.append("&quot;");
//      else if(c == ' ')
//        sb.append("&nbsp;");
      else
        sb.append(c);
    }
    return sb.toString();
  }

  /**
   * Describe <code>checkFIleSize</code> method : check File size String.
   * <br>t@CTCY̎w蕶`FbN
   * <br>R܂͐2ƏP̐K,M,Gw̏ꍇm[}ƂB
   * @param fs a <code>String</code> : `FbN镶
   * @return a <code>boolean</code> : G[̂Ƃ <code>true</code>
   */
  public static final boolean checkFIleSize(String fs) {
    Pattern pt = Pattern.compile("[0-9]{1,2}.?[0-9]?[kmgKMG]?");
    Matcher mt = pt.matcher(fs);
    return !mt.matches();
  }

  /**
   * Describe <code>convertFileSize</code> method : convert file size String to long.
   * <br>t@CTCY(xxxK, xxxM, xxxG)oCgɕϊB
   * <br>nnK -&gt; nn * 1024,  nnM -&gt; nn* 1024 * 1024 ߂
   * @param fs a <code>String</code> : t@CTCY̕
   * @return a <code>long</code> : ϊ̃oCg
   */
  public static final long convertFileSize(String fs) {
    int base = 1;
    StringBuffer sb = new StringBuffer(fs.toUpperCase());
    char lc = sb.charAt(sb.length()- 1);
    if (lc == 'K' || lc == 'M' || lc == 'G') {
      base = 1024;
      if (lc == 'M' || lc == 'G') {
        base *= 1024;
        if (lc == 'G') base *= 1024;
      }
      sb.setLength(sb.length()-1);
    }
    float fsz = Float.parseFloat(sb.toString());
    return (long)(fsz*base);
  }
  /**
   * Describe <code>convertFileSize</code> method : convert file size long to String.
   * <br>t@CTCỸoCg𕶎(xxxK, xxxM, xxxG)ɕϊB
   * <br>
   * @param fs a <code>long</code> : t@CTCY
   * @return a <code>String</code> : ϊ̕
   */
  public static final String convertFileSize(long sz) {
		if (sz < 1024) {
			return String.valueOf(sz);
		}
		if (sz < 1024 * 1024) {
			sz = Math.round((sz * 10) / 1024);
			return String.valueOf(sz / 10) + "K";
		}
		if (sz < 1024 * 1024 * 1024) {
			sz = Math.round((sz * 10) / (1024 * 1024));
			return String.valueOf(sz / 10) + "M";
		}
		sz = Math.round((sz * 10) / (1024 * 1024 * 1024));
		return String.valueOf(sz / 10) + "G";
  }

}
