package com.hamaf.hikyakucms.sample.schedule.ui;

import java.util.List;
import java.util.Map;
import com.hamaf.hikyakucms.sample.schedule.dto.VScheduleDto;
import com.hamaf.hikyakucms.sample.schedule.dto.VScheduleSearchDto;
import com.hamaf.hikyakucms.sample.schedule.process.ScheduleProcess;
import com.hamaf.hikyakucms.sample.schedule.process.TargetProcess;
import com.hamaf.hikyakucms.sample.schedule.ui.mapping.VScheduleMap;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.KeyList;
import com.hamaf.hikyaku.dataobject.RetrieveKeysInterface;
import com.hamaf.hikyaku.distribute.IllegalRetrieveDataException;
import com.hamaf.hikyaku.security.SecurityCheckUtils;
import com.hamaf.hikyaku.security.dto.UserDto;
import com.hamaf.hikyaku.ui.StandardListProcess;

/**
 * <p>^CgFRetrieveList is .</p>
 * <p>Fpꗗ쐬</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2011/02/27</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class RetrieveList extends StandardListProcess {

  private ScheduleProcess _sp;
	private TargetProcess _tp = null;
	private SecurityCheckUtils _scu = null;

	/**
	 * Creates a new <code>RetrieveList.java</code> instance.
	 * <br>Default Constructor
	 */
	public RetrieveList() {
    _sp = new ScheduleProcess();
		_tp = new TargetProcess();
		_scu = new SecurityCheckUtils();
    _jim = new VScheduleMap();
	}

	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#setRetrieveVal(com.hamaf.hikyaku.dataobject.KeyList)
	 */
	@Override
	protected RetrieveKeysInterface setRetrieveVal(KeyList keyl)
			throws IllegalRetrieveDataException {
    VScheduleSearchDto ssd = new VScheduleSearchDto();
    ssd.setRetrieveValue(VScheduleSearchDto.RETVAL, "%" + keyl.getRetrieve("key") + "%");
    ssd.setCondition(VScheduleSearchDto.RETVAL, "like");
    try {
      List<String> users = _tp.getUsers(getUserID(), sysId);
      StringBuilder sb = new StringBuilder();
      for (String usr : users) {
				sb.append("'");
				sb.append(usr);
				sb.append("',");
			}
      if (sb.length() > 1) sb.setLength(sb.length() - 1);
      ssd.setRetrieveValue(VScheduleSearchDto.USER_ID, sb.toString());
      ssd.setCondition(VScheduleSearchDto.USER_ID, "in");
		}
		catch (Exception ex) {
      ssd.setRetrieveValue(VScheduleSearchDto.USER_ID, getUserID());
		}
		ssd.setOrderItem(VScheduleSearchDto.DAY, -1);
		ssd.setOrderItem(VScheduleSearchDto.TIME_FROM, 2);
		ssd.setOrderItem(VScheduleSearchDto.TIME_TO, 3);
		ssd.setOrderItem(VScheduleSearchDto.USER_ID, 4);
		return ssd;
	}

	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#findKeys(com.hamaf.hikyaku.dataobject.RetrieveKeysInterface)
	 */
	@Override
	protected RetrieveKeysInterface findKeys(RetrieveKeysInterface rki)
			throws Exception {
    return _sp.find((VScheduleSearchDto)rki);
	}

	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#newDataObj()
	 */
	@Override
	protected DetailItemsInterface newDataObj() {
    return new VScheduleDto();
	}

	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
	 */
	@Override
	protected DetailItemsInterface callControl(DetailItemsInterface dii)
			throws Exception {
    return _sp.findviewbykey((VScheduleDto)dii, super.getUserID());
	}

	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#checkDelete(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
	 */
	@Override
	protected String[] checkDelete(DetailItemsInterface dii) {
		return null;
	}

	private final String MSG_NAME = "username";
	/* ( Javadoc)
	 * @see com.hamaf.hikyaku.ui.StandardListProcess#editItems(com.hamaf.hikyaku.dataobject.DetailItemsInterface, java.util.Map)
	 */
	@Override
	protected Map<String, String> editItems(DetailItemsInterface dii,
			Map<String, String> mp) throws Exception {
    UserDto ud = _scu.getUserInf(dii.getItemValue(VScheduleDto.USER_ID), getUserID());
    String rnm = ud.getItemValue(UserDto.ABBREV);
    if (rnm ==  null || rnm.length() == 0) {
    	mp.put(MSG_NAME, ud.getItemValue(UserDto.NAME));
    } else {
    	mp.put(MSG_NAME, rnm);
    }
		return super.editItems(dii, mp);
	}

}
