package com.hamaf.hikyakucms.sample.sns.process;

import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface.COMMFLD;
import com.hamaf.hikyaku.dataobject.PreparedItems;
import com.hamaf.hikyaku.security.SecurityCheckUtils;
import com.hamaf.hikyaku.security.dto.UserDto;
import com.hamaf.hikyaku.util.DateUtil;
import com.hamaf.hikyakucms.sample.sns.access.ConversationAccess;
import com.hamaf.hikyakucms.sample.sns.access.DeleteConversationAccess;
import com.hamaf.hikyakucms.sample.sns.access.Hatsuban2Access;
import com.hamaf.hikyakucms.sample.sns.access.SqlExec;
import com.hamaf.hikyakucms.sample.sns.access.Hatsuban2Access.HATSUBAN2_KBN;
import com.hamaf.hikyakucms.sample.sns.dto.ConversationDto;
import com.hamaf.hikyakucms.sample.sns.dto.ConversationSearchDto;

/**
 * <p>^CgFConversationProcess is .</p>
 * <p>F</p>
 * <br>b^pNX
 *
 * <p>쌠: Copyright (c) 2011/03/23</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class ConversationProcess {

  private ConversationAccess _ca = null;
  private SecurityCheckUtils _scu = null;
  private Hatsuban2Access _ha = null;
  private SqlExec _se = null;

  /**
   * Creates a new <code>ConversationProcess.java</code> instance.
   * <br>Default Constructor
   */
  public ConversationProcess() {
    _ca = new ConversationAccess();
    _scu = new SecurityCheckUtils();
    _ha = new Hatsuban2Access();
    _se = new SqlExec();

//    _ca.setDebug();
//    _se.setDebug();
  }

  /**
   * Describe <code>find</code> method : .
   * <br>b^̌B
   *
   * @param csd a <code>ConversationSearchDto</code> :
   * @return a <code>ConversationSearchDto</code> :
   * @throws Exception
   */
  public ConversationSearchDto find(ConversationSearchDto csd) throws Exception {
    return (ConversationSearchDto)_ca.find(csd);
  }

  /**
   * Describe <code>findByPrimaryKey</code> method : .
   * <br>b^̏ڍ׏擾B
   *
   * @param cd a <code>ConversationDto</code> :
   * @return a <code>ConversationDto</code> :
   * @throws Exception
   */
  public ConversationDto findByPrimaryKey(ConversationDto cd) throws Exception {
    cd.clearRequest(ConversationDto.ENTRY_NAME);
    cd.clearRequest(ConversationDto.ICON);
    cd = (ConversationDto)_ca.findByPrimaryKey(cd);
    String uid = cd.getItemValue(ConversationDto.ENTRY_USER);
    UserDto ud = _scu.getUserInf(uid, cd.getCommFld(COMMFLD.UUSR));
    String unm = ud.getItemValue(UserDto.ABBREV);
    if (unm == null || unm.length() == 0) unm = ud.getItemValue(UserDto.NAME);
    cd.setItemValue(ConversationDto.ENTRY_NAME, unm);
    cd.setItemValue(ConversationDto.ICON, ud.getItemValue(UserDto.ICON_UMU));
    return cd;
  }

  /**
   * Describe <code>add</code> method : .
   * <br>b^̓o^B
   *
   * @param cd a <code>ConversationDto</code> :
   * @throws Exception
   */
  public void add(ConversationDto cd) throws Exception {
    String uid = cd.getCommFld(COMMFLD.UUSR);
    cd.setItemValue(ConversationDto.ENTRY_USER, uid);
    cd.clearRequest(ConversationDto.ENTRY_NAME);
    cd.clearRequest(ConversationDto.ICON);
    cd.setItemValue(ConversationDto.ENTRY_DATE, DateUtil.getDate());
    String seq = _ha.getNextNo(HATSUBAN2_KBN.CONVERSATION, uid, cd.getSysId());
    cd.setItemValue(ConversationDto.SEQNO,seq);
    String lvl = cd.getParLvl();
    if (lvl == null || lvl.length() == 0) {
      cd.setItemValue(ConversationDto.LEVEL, "1");
      cd.setItemValue(ConversationDto.ORIGINAL_SEQ, seq);
      long isq = 999999999 - Integer.parseInt(seq);
      DecimalFormat df = new DecimalFormat("000000000");
      cd.setItemValue(ConversationDto.SEQUENCE, df.format(isq));
    } else {
      int ilv = Integer.parseInt(lvl);
      cd.setItemValue(ConversationDto.LEVEL, String.valueOf(ilv + 1));
      StringBuilder sb = new StringBuilder();
      sb.append(cd.getParSeq());
      sb.append(seq);
      cd.setItemValue(ConversationDto.SEQUENCE, sb.toString());
    }
    _ca.regist(cd);
  }

  /**
   * Describe <code>mod</code> method : .
   * <br>b^̍XVB
   *
   * @param cd a <code>ConversationDto</code> :
   * @throws Exception
   */
  public void mod(ConversationDto cd) throws Exception {
    cd.clearRequest(ConversationDto.ENTRY_NAME);
    cd.clearRequest(ConversationDto.ICON);
    _ca.update(cd);
  }

  /**
   * Describe <code>modFinish</code> method : .
   * <br>ݒ肷B
   *
   * @param cd a <code>ConversationDto</code> :
   * @throws Exception
   */
  public void modFinish(ConversationDto cd) throws Exception {
    PreparedItems pi = new PreparedItems();
    pi.setSysId(cd.getSysId());
    Map<String, Object> pm = new HashMap<String, Object>();
    pm.put("FINISH", DateUtil.getDate());
    pm.put("ORIGINAL", cd.getItemValue(ConversationDto.SEQNO));
    pi.addParam(pm);
    _se.setFinishDay(pi);
  }

  /**
   * Describe <code>del</code> method : .
   * <br>b^̍폜B
   *
   * @param cd a <code>ConversationDto</code> :
   * @throws Exception
   */
  public void del(ConversationDto cd) throws Exception {
    DeleteConversationAccess dca = new DeleteConversationAccess();
    try {
      delChild(cd.getItemValue(ConversationDto.SEQNO), cd.getSysId(), dca);
    }
    catch (Exception ex) {
      dca.rollBack();
      throw ex;
    }
    dca.commit();
  }
  private void delChild(String seqNo, String sysId,
      DeleteConversationAccess dca) throws Exception {
    ConversationSearchDto csd = new ConversationSearchDto();
    csd.setRetrieveValue(ConversationDto.PARENT_SEQ, seqNo);
    csd.setSysId(sysId);
    csd = (ConversationSearchDto)_ca.find(csd);
    for (List<String> keys : csd) {
      delChild(keys.get(0), sysId, dca);
    }
    ConversationDto cd = new ConversationDto();
    cd.setItemValue(ConversationDto.SEQNO, seqNo);
    cd.setSysId(sysId);
    cd = (ConversationDto)_ca.findByPrimaryKey(cd);
    dca.delConversation(cd);
  }
}
