package com.hamaf.hikyakucms.sample.sns.process;

import java.util.HashMap;
import java.util.Map;
import com.hamaf.hikyaku.dataobject.PreparedItems;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface.COMMFLD;
import com.hamaf.hikyaku.security.SecurityCheckUtils;
import com.hamaf.hikyaku.security.dto.UserDto;
import com.hamaf.hikyakucms.sample.sns.access.Hatsuban2Access;
import com.hamaf.hikyakucms.sample.sns.access.NoticeAccess;
import com.hamaf.hikyakucms.sample.sns.access.SqlExec;
import com.hamaf.hikyakucms.sample.sns.access.Hatsuban2Access.HATSUBAN2_KBN;
import com.hamaf.hikyakucms.sample.sns.dto.NoticeDto;
import com.hamaf.hikyakucms.sample.sns.dto.NoticeSearchDto;

/**
 * <p>^CgFNoticeProcess is .</p>
 * <p>Fm点ʒm</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2011/03/06</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class NoticeProcess {

  private NoticeAccess _na = null;
  private Hatsuban2Access _ha = null;
  private SecurityCheckUtils _scu = null;
  private SqlExec _se = null;

  /**
   * Creates a new <code>NoticeProcess.java</code> instance.
   * <br>Default Constructor
   */
  public NoticeProcess() {
    _na = new NoticeAccess();
    _ha = new Hatsuban2Access();
    _scu = new SecurityCheckUtils();
    _se = new SqlExec();

//    _na.setDebug();
//    _se.setDebug();
  }

  /**
   * Describe <code>find</code> method : .
   * <br>m点ʒm̌B
   *
   * @param nsd a <code>NoticeSearchDto</code> :
   * @return a <code>NoticeSearchDto</code> :
   * @throws Exception
   */
  public NoticeSearchDto find(NoticeSearchDto nsd) throws Exception {
    return (NoticeSearchDto)_na.find(nsd);
  }

  /**
   * Describe <code>findByPrimaryKey</code> method : .
   * <br>m点ʒm̏ڍ׏擾B
   *
   * @param nd a <code>NoticeDto</code> :
   * @return a <code>NoticeDto</code> :
   * @throws Exception
   */
  public NoticeDto findByPrimaryKey(NoticeDto nd ) throws Exception {
    nd.clearRequest(NoticeDto.ENTRY_USER_NAME);
    nd = (NoticeDto)_na.findByPrimaryKey(nd);
    String uid = nd.getItemValue(NoticeDto.ENTRY_USER);
    UserDto ud = _scu.getUserInf(uid, nd.getCommFld(COMMFLD.UUSR));
    String unm = ud.getItemValue(UserDto.ABBREV);
    if (unm == null || unm.length() == 0) unm = ud.getItemValue(UserDto.NAME);
    nd.setItemValue(NoticeDto.ENTRY_USER_NAME, unm);
    return nd;
  }

  /**
   * Describe <code>add</code> method : .
   * <br>m点ʒm̓o^B
   *
   * @param nd a <code>NoticeDto</code> :
   * @throws Exception
   */
  public void add(NoticeDto nd) throws Exception {
    String uid = nd.getItemValue(NoticeDto.USER_ID);
    String euid = nd.getItemValue(NoticeDto.ENTRY_USER);
    if (uid == null || uid.length() == 0) throw new IllegalArgumentException("UserID is null");
    UserDto ud = _scu.getUserInf(uid, euid);
    if (ud.isNotFound()) throw new IllegalArgumentException("UserID (" + uid + ") is not found");

    if (euid == null || euid.length() == 0) throw new IllegalArgumentException("EntryUserID is null");
    UserDto eud = _scu.getUserInf(euid, euid);
    if (eud.isNotFound()) throw new IllegalArgumentException("EntryUserID (" + euid + ") is not found");

    nd.clearRequest(NoticeDto.ENTRY_USER_NAME);
    nd.setItemValue(NoticeDto.SEQNO,
        _ha.getNextNo(HATSUBAN2_KBN.NOTICE, uid, nd.getSysId()));
    _na.regist(nd);
  }

  /**
   * Describe <code>mod</code> method : .
   * <br>m点ʒm̍XVB
   *
   * @param nd a <code>NoticeDto</code> :
   * @throws Exception
   */
  public void mod(NoticeDto nd) throws Exception {
    nd.clearRequest(NoticeDto.ENTRY_USER_NAME);
    _na.updateNoExclusion(nd);
  }

  /**
   * Describe <code>del</code> method : .
   * <br>m点ʒm̍폜B
   *
   * @param nd a <code>NoticeDto</code> :
   * @throws Exception
   */
  public void del(NoticeDto nd) throws Exception {
    _na.deleteNoExclusion(nd);
  }

  /**
   * Describe <code>delAll</code> method : .
   * <br>w肳ꂽ[U[ŊmFwȑÕf[^폜B
   *
   * @param uid a <code>String</code> : [U[ID
   * @param fdate a <code>String</code> : mF
   * @param sysId a <code>String</code> : vOCID
   * @throws Exception
   */
  public void delAll(String uid, String fdate, String sysId) throws Exception {
    PreparedItems pi = new PreparedItems();
    pi.setSysId(sysId);
    Map<String, Object> pm = new HashMap<String, Object>();
    pm.put("USERID", uid);
    pm.put("AFDATE", fdate);
    pi.addParam(pm);
    _se.delNotice(pi);
  }
}
