package com.hamaf.hikyakucms.sample.sns.ui;

import java.util.List;
import com.hamaf.hikyaku.dataobject.CommonMessage;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface.COMMFLD;
import com.hamaf.hikyaku.security.SecurityCheckUtils;
import com.hamaf.hikyaku.security.dto.UserDto;
import com.hamaf.hikyaku.ui.StandardDetailProcess;
import com.hamaf.hikyakucms.sample.sns.access.Hatsuban2Access.HATSUBAN2_KBN;
import com.hamaf.hikyakucms.sample.sns.dto.VCommunicationDto;
import com.hamaf.hikyakucms.sample.sns.process.CommunicationProcess;
import com.hamaf.hikyakucms.sample.sns.process.FootprintsProcess;
import com.hamaf.hikyakucms.sample.sns.process.RestrictionProcess;
import com.hamaf.hikyakucms.sample.sns.ui.mapping.VCommunicationMap;

/**
 * <p>^CgFCommunicationDetail is .</p>
 * <p>FȀڍ׉ʕ\</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2011/03/05</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class CommunicationDetail extends StandardDetailProcess {

  public static final String MSG_GID = "groupId";

  private CommunicationProcess _cp = null;
  private FootprintsProcess _fp = null;
  private RestrictionProcess _rp = null;
  private SecurityCheckUtils _scu = null;

  /**
   * Creates a new <code>CommunicationDetail.java</code> instance.
   * <br>Default Constructor
   */
  public CommunicationDetail() {
    _jim = new VCommunicationMap();
    _cp = new CommunicationProcess();
    _fp = new FootprintsProcess();
    _rp = new RestrictionProcess();
    _scu = new SecurityCheckUtils();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardDetailProcess#newDataObj()
   */
  @Override
  protected DetailItemsInterface newDataObj() {
    return new VCommunicationDto();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardDetailProcess#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected DetailItemsInterface callControl(DetailItemsInterface dii) throws Exception {
    return _cp.getDetail((VCommunicationDto)dii);
  }

  private final String MSG_FP = "footprints";
  private final String MSG_UP = "updateFlg";
  private final String MSG_MD = "managerFlg";
  private final String MSG_MN = "managerName";
  private final String MSG_GNM = "groupName";
  private final String MSG_KBN = "docKbn";
  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardDetailProcess#editItems(com.hamaf.hikyaku.dataobject.CommonMessage)
   */
  @Override
  protected CommonMessage editItems(CommonMessage msg) throws Exception {
    if (msg.getResMsg("dayFrom").startsWith("0001")) msg.setResMsg("dayFrom", "");
    if (msg.getResMsg("dayTo").startsWith("9999")) msg.setResMsg("dayTo", "");

    msg.setResMsg(MSG_KBN, HATSUBAN2_KBN.COMMUNICATION.getKey());
    String uid = getUserID();
    msg.setResMsg(MSG_FP,
        _fp.isFootprints(HATSUBAN2_KBN.COMMUNICATION, msg.getResMsg("seqNo"), sysId, uid));
    String mgr = msg.getResMsg("manager");
    UserDto ud = _scu.getUserInf(mgr, uid);
    String mnm = ud.getItemValue(UserDto.ABBREV);
    if (mnm == null || mnm.length() == 0) mnm = ud.getItemValue(UserDto.NAME);
    msg.setResMsg(MSG_MN, mnm);

    if (uid.equals(mgr)) {
      msg.setResMsg(MSG_MD, "1");
      msg.setResMsg(MSG_UP, "1");
    } else {
      msg.setResMsg(MSG_MD, "0");
      if (uid.equals(msg.getResMsg(COMMFLD.UUSR.getVal()))) {
        msg.setResMsg(MSG_UP, "1");
      } else {
        msg.setResMsg(MSG_UP, "0");
      }
    }
    msg = setGroupInf(msg);
    return super.editItems(msg);
  }
  private CommonMessage setGroupInf(CommonMessage msg) throws Exception {
    String au = msg.getResMsg("allFlg");
    if ("1".equals(au)) {
      msg.setResMsg(MSG_GNM, "S");
      return msg;
    }

    List<String[]> gdts = _rp.getGroups(HATSUBAN2_KBN.COMMUNICATION, msg.getResMsg("seqNo"), sysId, getUserID());
    StringBuilder idsb = new StringBuilder();
    StringBuilder nmsb = new StringBuilder();
    for (String[] gpdt : gdts) {
      idsb.append(gpdt[0]);
      idsb.append(",");
      nmsb.append(gpdt[1]);
      nmsb.append(",");
    }
    if (idsb.length() > 0) {
      idsb.setLength(idsb.length() - 1);
      nmsb.setLength(nmsb.length() - 1);
    }
    msg.setResMsg(MSG_GID, idsb.toString());
    msg.setResMsg(MSG_GNM, nmsb.toString());
    return msg;
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardDetailProcess#setKeys(com.hamaf.hikyaku.dataobject.DetailItemsInterface, com.hamaf.hikyaku.dataobject.CommonMessage)
   */
  @Override
  protected boolean setKeys(DetailItemsInterface dii, CommonMessage msg) {
    String ps = msg.getReqStr("pos");
    String seq = msg.getReqStr("seqNo");
    if (ps.length() > 0) return false;
    dii.setItemValue(VCommunicationDto.SEQNO, seq);
    return true;
  }

}
