package com.hamaf.hikyakucms.sample.sns.ui;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.KeyList;
import com.hamaf.hikyaku.dataobject.RetrieveKeysInterface;
import com.hamaf.hikyaku.distribute.IllegalRetrieveDataException;
import com.hamaf.hikyaku.ui.StandardListProcess;
import com.hamaf.hikyakucms.sample.sns.dto.ConversationDto;
import com.hamaf.hikyakucms.sample.sns.dto.ConversationSearchDto;
import com.hamaf.hikyakucms.sample.sns.process.ConversationProcess;
import com.hamaf.hikyakucms.sample.sns.ui.mapping.ConversionMap;

/**
 * <p>^CgFConversationList is .</p>
 * <p>Fb^̈ꗗ\</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2011/03/23</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class ConversationList extends StandardListProcess {

  private ConversationProcess _cp = null;

  /**
   * Creates a new <code>ConversationList.java</code> instance.
   * <br>Default Constructor
   */
  public ConversationList() {
    _jim = new ConversionMap();
    _cp = new ConversationProcess();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#setRetrieveVal(com.hamaf.hikyaku.dataobject.KeyList)
   */
  @Override
  protected RetrieveKeysInterface setRetrieveVal(KeyList keyl)
      throws IllegalRetrieveDataException {
    ConversationSearchDto csd = new ConversationSearchDto();
    String sval = keyl.getRetrieve("key");
    if (!"1".equals(keyl.getRetrieve("sfinish")) && sval.length() == 0) {
      csd.setRetrieveValue(ConversationSearchDto.FINISH_DATE, "NULL");
      csd.setCondition(ConversationSearchDto.FINISH_DATE, "IS");
    }
    String ttl = keyl.getRetrieve("stitle");
    if (ttl.length() == 0) ttl = sval;
    if (ttl.length() > 0) {
      csd.setRetrieveValue(ConversationSearchDto.TITLE, "%" + ttl + "%");
      csd.setCondition(ConversationSearchDto.TITLE, "LIKE");
    }
    csd.setRetrieveValue(ConversationSearchDto.LEVEL, "1");
    csd.setOrderItem(ConversationSearchDto.SEQUENCE, 1);
    if (!"1".equals(keyl.getRetrieve("dispchild"))) return csd;

    csd.setSysId(sysId);
    try {
      csd = (ConversationSearchDto)_cp.find(csd);
    }
    catch (Exception ex) {
      throw new IllegalRetrieveDataException(ex.toString(), new HashMap<String, String>());
    }
    StringBuilder sb = new StringBuilder();
    for (List<String> keys : csd) {
      sb.append("'");
      sb.append(keys.get(0));
      sb.append("',");
    }
    if (sb.length() == 0) return csd;

    ConversationSearchDto csd2 = new ConversationSearchDto();
    sb.setLength(sb.length() - 1);
    csd2.setRetrieveValue(ConversationSearchDto.ORIGINAL_SEQ, sb.toString());
    csd2.setCondition(ConversationSearchDto.ORIGINAL_SEQ, "IN");
    csd2.setOrderItem(ConversationSearchDto.SEQUENCE, 1);
    return csd2;
  }

  private final String MSG_MGN = "leftmargin";
  private final String MSG_LST = "displist";
  private final String MSG_END = "dispfinish";
  private final String MSG_REP = "dispreplay";
  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#editItems(com.hamaf.hikyaku.dataobject.DetailItemsInterface, java.util.Map)
   */
  @Override
  protected Map<String, String> editItems(DetailItemsInterface dii,
      Map<String, String> mp) throws Exception {
    int ilv = Integer.parseInt(dii.getItemValue(ConversationDto.LEVEL));
    ilv --;
    mp.put(MSG_MGN, String.valueOf(ilv * 15));
    String fdt = dii.getItemValue(ConversationDto.FINISH_DATE);
    if (fdt == null || fdt.length() == 0) {
      mp.put(MSG_REP, "ԐM");
    } else {
      mp.put(MSG_REP, "");
    }

    if (ilv == 0) {
      mp.put(MSG_LST, "ꗗ");
      String flg = dii.getItemValue(ConversationDto.FINISH_DATE);
      if (flg == null || flg.length() == 0) {
        mp.put(MSG_END, "");
      } else {
        mp.put(MSG_END, "");
      }
      return mp;
    }

    mp.put(MSG_END, "");
    mp.put(MSG_LST, "");
    String dtl = dii.getItemValue(ConversationDto.DETAIL);
    if (dtl.length() > 50) dtl = dtl.substring(0, 50) + " ...";
    mp.put("title", dtl);
    return super.editItems(dii, mp);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#findKeys(com.hamaf.hikyaku.dataobject.RetrieveKeysInterface)
   */
  @Override
  protected RetrieveKeysInterface findKeys(RetrieveKeysInterface rki)
      throws Exception {
    return _cp.find((ConversationSearchDto)rki);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#newDataObj()
   */
  @Override
  protected DetailItemsInterface newDataObj() {
    return new ConversationDto();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected DetailItemsInterface callControl(DetailItemsInterface dii)
      throws Exception {
    return _cp.findByPrimaryKey((ConversationDto)dii);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#checkDelete(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected String[] checkDelete(DetailItemsInterface dii) {
    return null;
  }

}
