package com.hamaf.hikyakucms.sample.sns.ui;

import java.util.Map;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.KeyList;
import com.hamaf.hikyaku.dataobject.RetrieveKeysInterface;
import com.hamaf.hikyaku.distribute.IllegalRetrieveDataException;
import com.hamaf.hikyaku.ui.StandardListProcess;
import com.hamaf.hikyakucms.sample.sns.dto.NoticeDto;
import com.hamaf.hikyakucms.sample.sns.dto.NoticeSearchDto;
import com.hamaf.hikyakucms.sample.sns.process.NoticeProcess;
import com.hamaf.hikyakucms.sample.sns.ui.mapping.NoticeMap;

/**
 * <p>^CgFNoticeList is .</p>
 * <p>Fm点ʒmꗗ</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2011/03/07</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class NoticeList extends StandardListProcess {

  private NoticeProcess _np = null;

  /**
   * Creates a new <code>NoticeList.java</code> instance.
   * <br>Default Constructor
   */
  public NoticeList() {
    _jim = new NoticeMap();
    _np = new NoticeProcess();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#setRetrieveVal(com.hamaf.hikyaku.dataobject.KeyList)
   */
  @Override
  protected RetrieveKeysInterface setRetrieveVal(KeyList keyl) throws IllegalRetrieveDataException {
    NoticeSearchDto nsd = new NoticeSearchDto();
    nsd.setRetrieveValue(NoticeSearchDto.USER_ID, getUserID());
    String rdt = keyl.getRetrieve("sentryUser");
    if (rdt != null && rdt.length() > 0) {
      nsd.setRetrieveValue(NoticeSearchDto.ENTRY_USER, rdt);
      nsd.setCondition(NoticeSearchDto.ENTRY_USER, "IN");
    }
    rdt = keyl.getRetrieve("sentryDay");
    if (rdt != null && rdt.length() > 0) {
      nsd.setRetrieveValue(NoticeSearchDto.ENTRY_DAY, rdt);
      nsd.setCondition(NoticeSearchDto.ENTRY_DAY, "<=");
    }
    rdt = keyl.getRetrieve("saffirmationDay");
    if ("1".equals(keyl.getRetrieve("rtyp")) && !"1".equals(keyl.getRetrieve("snocheck"))) {
      if (rdt != null && rdt.length() > 0) {
        StringBuilder sb2 = new StringBuilder();
        sb2.append("'0000/01/01' AND '");
        sb2.append(rdt);
        sb2.append("'");
        nsd.setRetrieveValue(NoticeSearchDto.AFFIRMATION_DAY, sb2.toString());
        nsd.setCondition(NoticeSearchDto.AFFIRMATION_DAY, "BETWEEN");
      }
    } else {
      nsd.setRetrieveValue(NoticeSearchDto.AFFIRMATION_DAY, "0000/00/00");
    }
    rdt = keyl.getRetrieve("scontents");
    if (rdt != null && rdt.length() > 0) {
      nsd.setRetrieveValue(NoticeSearchDto.CONTENTS, "%" + rdt + "%");
      nsd.setCondition(NoticeSearchDto.CONTENTS, "LIKE");
    }

    nsd.setOrderItem(NoticeSearchDto.ENTRY_DAY, -1);
    nsd.setOrderItem(NoticeSearchDto.AFFIRMATION_DAY, 2);
    nsd.setOrderItem(NoticeSearchDto.USER_ID, 3);

    return nsd;
  }

  private final String MSG_DTL = "detail";
  private final String MSG_PRM = "parameter";
  private final String MSG_MOD = "modify";
  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#editItems(com.hamaf.hikyaku.dataobject.DetailItemsInterface, java.util.Map)
   */
  @Override
  protected Map<String, String> editItems(DetailItemsInterface dii, Map<String, String> mp) throws Exception {
    if (dii.getItemValue(NoticeDto.PLUGIN_ID).length() > 0 &&
        dii.getItemValue(NoticeDto.PROCESS_ID).length() > 0 &&
        dii.getItemValue(NoticeDto.ACTION_ID).length() > 0) {
      mp.put(MSG_DTL, "ڍ");
    } else {
      mp.put(MSG_DTL, "");
    }
    StringBuilder sb = new StringBuilder();
    sb.append("'");
    sb.append(dii.getItemValue(NoticeDto.PLUGIN_ID));
    sb.append("','");
    sb.append(dii.getItemValue(NoticeDto.PROCESS_ID));
    sb.append("','");
    sb.append(dii.getItemValue(NoticeDto.ACTION_ID));
    sb.append("','");
    sb.append(dii.getItemValue(NoticeDto.KEY_VALUES));
    sb.append("'");
    mp.put(MSG_PRM, sb.toString());
    if ("0000/00/00".equals(dii.getItemValue(NoticeDto.AFFIRMATION_DAY))) {
      mp.put(MSG_MOD, "ς");
    } else {
      mp.put(MSG_MOD, "");
    }
    return super.editItems(dii, mp);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#findKeys(com.hamaf.hikyaku.dataobject.RetrieveKeysInterface)
   */
  @Override
  protected RetrieveKeysInterface findKeys(RetrieveKeysInterface rki) throws Exception {
    return _np.find((NoticeSearchDto)rki);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#newDataObj()
   */
  @Override
  protected DetailItemsInterface newDataObj() {
    return new NoticeDto();
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#callControl(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected DetailItemsInterface callControl(DetailItemsInterface dii) throws Exception {
    return _np.findByPrimaryKey((NoticeDto)dii);
  }

  /* ( Javadoc)
   * @see com.hamaf.hikyaku.ui.StandardListProcess#checkDelete(com.hamaf.hikyaku.dataobject.DetailItemsInterface)
   */
  @Override
  protected String[] checkDelete(DetailItemsInterface dii) {
    if (delCheck(dii)) {
      String[] ret = {"contents", "### 폜f[^ł"};
      return ret;
    }
    return null;
  }

}
