﻿package com.hamaf.hikyakucms.sample.wiki.access;

import java.sql.SQLException;

import com.hamaf.hikyaku.access.DefaultTableAccess;
import com.hamaf.hikyaku.access.ExclusionException;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface.COMMFLD;
import com.hamaf.hikyaku.util.DateUtil;
import com.hamaf.hikyaku.util.TextUtil;
import com.hamaf.hikyakucms.sample.wiki.dto.WikiDocDto;

/**
 * <p>タイトル：WikiDocAccess is .</p>
 * <p>説明：M_WIKIDOC(Wiki文書管理)アクセス用クラス</p>
 * <br>
 *
 * <p>著作権: Copyright ©2012/09/06</p>
 * @author @USR
 * @version 1.0
 */
public class WikiDocAccess extends DefaultTableAccess {

  private final String TBLNM = "M_WIKIDOC";
  private final String PARM = "hsmpl";

  /**
   *  Creates a new <code>WikiDocAccess.java</code> instance.
   * <br>Default Constructor
   */
  public WikiDocAccess() {
    tableName = TBLNM;
    connectionParameter = PARM;
    super.setDebug();
  }

  @Override
  public void regist(DetailItemsInterface dii) throws SQLException {
    dii = dtoComplation(dii, true);
    super.regist(dii);
  }

  private DetailItemsInterface dtoComplation(DetailItemsInterface di, boolean addNew) {
    if (addNew) {
      di.setItemValue(WikiDocDto.CREATE_DATE, DateUtil.getDate());
      di.setItemValue(WikiDocDto.CREATE_TIME, DateUtil.getTime());
      di.setItemValue(WikiDocDto.AUTHOR, di.getCommFld(COMMFLD.UUSR));
    }
    di.setItemValue(WikiDocDto.LAST_DATE, DateUtil.getDate());
    di.setItemValue(WikiDocDto.LAST_TIME, DateUtil.getTime());
    di.setItemValue(WikiDocDto.LAST_AUTHOR, di.getCommFld(COMMFLD.UUSR));
    if (di.isRequestItem(WikiDocDto.TITLE)) {
      StringBuilder sb = new StringBuilder();
      sb.append(di.getItemValue(WikiDocDto.TITLE));
      sb.append("\t");
      sb.append(di.getItemValue(WikiDocDto.SAMMARY));
      String rdt = sb.toString().toUpperCase();
      rdt = TextUtil.cnv2Zen(rdt);
      rdt = TextUtil.kanaHan2Zen(rdt);
      rdt = TextUtil.cnvKata2Hira(rdt);
      di.setItemValue(WikiDocDto.RETVAL, rdt);
    }
    return di;
  }

  @Override
  public void update(DetailItemsInterface dii) throws ExclusionException,
      SQLException {
    dii = dtoComplation(dii, false);
    super.update(dii);
  }

  @Override
  public void updateNoExclusion(DetailItemsInterface dii) throws SQLException {
    dii = dtoComplation(dii, false);
    super.updateNoExclusion(dii);
  }

}
