package com.hamaf.hikyakucms.sample.wiki.process;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.hamaf.hikyaku.dataobject.DetailItemsInterface.COMMFLD;
import com.hamaf.hikyaku.dataobject.PreparedItems;
import com.hamaf.hikyaku.dataobject.utils.CopyDtoUtil;
import com.hamaf.hikyaku.security.SecurityCheckUtils;
import com.hamaf.hikyaku.security.dto.UserDto;
import com.hamaf.hikyakucms.sample.wiki.access.HatsubanAccess;
import com.hamaf.hikyakucms.sample.wiki.access.HatsubanAccess.HATSUBAN_KBN;
import com.hamaf.hikyakucms.sample.wiki.access.SqlExec;
import com.hamaf.hikyakucms.sample.wiki.access.WikiArticleAccess;
import com.hamaf.hikyakucms.sample.wiki.access.WikiDiffAccess;
import com.hamaf.hikyakucms.sample.wiki.access.WikiDocAccess;
import com.hamaf.hikyakucms.sample.wiki.dto.DocArctileDtoMapping;
import com.hamaf.hikyakucms.sample.wiki.dto.DocDiffDtoMapping;
import com.hamaf.hikyakucms.sample.wiki.dto.WikiArticleDto;
import com.hamaf.hikyakucms.sample.wiki.dto.WikiArticleSearchDto;
import com.hamaf.hikyakucms.sample.wiki.dto.WikiDiffDto;
import com.hamaf.hikyakucms.sample.wiki.dto.WikiDiffSearchDto;
import com.hamaf.hikyakucms.sample.wiki.dto.WikiDocDto;
import com.hamaf.hikyakucms.sample.wiki.dto.WikiDocSearchDto;
import com.hamaf.hikyakucms.sample.wiki.util.WikiArticle;
import com.hamaf.hikyakucms.sample.wiki.util.WikiDiff;
import com.hamaf.hikyakucms.sample.wiki.util.WikiDocument;

/**
 * <p>^CgFWikiDocProcess is .</p>
 * <p>FWikȉNX</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2012/09/09</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class WikiDocProcess {

  private WikiDocAccess _wda = null;
  private WikiArticleAccess _waa  = null;
  private HatsubanAccess _ha = null;
  private SqlExec _sqe = null;
  private WikiDiffAccess _wdifa = null;
  private SecurityCheckUtils _scu = null;

  public WikiDocProcess() {
    _wda = new WikiDocAccess();
    _waa = new WikiArticleAccess();
    _ha = new HatsubanAccess();
    _sqe = new SqlExec();
    _wdifa = new WikiDiffAccess();
    _scu = new SecurityCheckUtils();
  }

  /**
   * Describe <code>addNew</code> method : add new document.
   * <br>VKWiki̓o^B
   * <ol>
   *   <li>ԍ𔭔ԂADB֓o^
   *   <li>L̓eLDB֓o^
   * </ol>
   *
   * @param wdd a <code>WikiDocDto</code>
   * @return a <code>String</code> : o^̕ԍ
   * @throws Exception
   */
  public String addNew(WikiDocDto wdd) throws Exception {
    String wno = _ha.getNextNo(HATSUBAN_KBN.WIKIDOC, null, wdd.getSysId());
    wdd.setItemValue(WikiDocDto.WIKI_NO, wno);
    _wda.regist(wdd);

    WikiDocument wd = new WikiDocument();
    wd.setDocument(wdd.getDocument(), wdd.getChapter(), wdd.getSection());
    WikiArticleDto wad = (WikiArticleDto)CopyDtoUtil.newDto(new DocArctileDtoMapping(), wdd);
    addText(wd.getDoc(), wad);
    return wno;
  }

  private void addText(WikiArticle wa, WikiArticleDto wad) throws Exception {
    wad.setItemValue(WikiArticleDto.SUB_TITLE, wa.getSubTitle());
    wad.setItemValue(WikiArticleDto.CHAPTER, String.valueOf(wa.getChapter()));
    wad.setItemValue(WikiArticleDto.SECTION, String.valueOf(wa.getSection()));
    wad.setItemValue(WikiArticleDto.PLAIN_TEXT, wa.getOriginal());
    wad.setItemValue(WikiArticleDto.HTML_TEXT, wa.getHtml());
    _waa.regist(wad);

    for (WikiArticle child : wa.getChildren()) {
      addText(child, wad);
    }
  }

  /**
   * Describe <code>editDoc</code> method : edit document.
   * <br>Wiki̍XVB
   * <ol>
   *   <li>DBXV
   *   <li>ύXDiffDB֓o^
   *   <li>LŋLDBւ
   *     <ul>
   *       <li>o^ς݂̋LXV镶ԍÁA߂ō폜
   *       <li>LVKɓo^
   *     </ul>
   * </ol>
   *
   * @param wdd a <code>WikiDocDto</code>
   * @throws Exception
   */
  public void editDoc(WikiDocDto wdd) throws Exception {
    _wda.update(wdd);

    WikiDocument wd = new WikiDocument();
    wd.setHeader(wdd.getItemValue(WikiDocDto.TITLE),
        wdd.getItemValue(WikiDocDto.CATEGORY), wdd.getItemValue(WikiDocDto.SAMMARY));
    wd.setDocument(wdd.getDocument(), wdd.getChapter(), wdd.getSection());
    wdd.setWikiDoc(wd);
    addDiff(wdd);

    PreparedItems pi = createPrepared(wdd);
    _sqe.delArticle(pi);
    WikiArticleDto wad = (WikiArticleDto)CopyDtoUtil.newDto(new DocArctileDtoMapping(), wdd);
    addText(wd.getDoc(), wad);
  }

  private PreparedItems createPrepared(WikiDocDto wdd) {
    PreparedItems pi = new PreparedItems();
    pi.setSysId(wdd.getSysId());
    Map<String, Object> prm = new HashMap<String, Object>();
    prm.put("WIKI_NO", wdd.getItemValue(WikiDocDto.WIKI_NO));
    if ("0".equals(wdd.getChapter())) {
      pi.addDelLine("C");
      pi.addDelLine("S");
    }  else {
      prm.put("CHAPTER", new Integer(wdd.getChapter()));
      if ("0".equals(wdd.getSection())) {
        pi.addDelLine("S");
      } else {
        prm.put("SECTION", new Integer(wdd.getSection()));
      }
    }
    pi.addParam(prm);
    return pi;
  }

  private void addDiff(WikiDocDto wdd) throws Exception {
    String wno = wdd.getItemValue(WikiDocDto.WIKI_NO);
    WikiDocDto old = new WikiDocDto();
    old.setItemValue(WikiDocDto.WIKI_NO, wno);
    old.setSysId(wdd.getSysId());
    old.setAllRequest();
    old.setChapter(wdd.getChapter());
    old.setSection(wdd.getSection());
    old = getDetailAll(old);

    String txt = WikiDiff.diffStr(wdd.getWikiDoc().getDiffDoc(),
        old.getWikiDoc().getDiffDoc());

    WikiDiffDto wdifd = (WikiDiffDto)CopyDtoUtil.newDto(new DocDiffDtoMapping(), wdd);
    String dno = _ha.getNextNo(HATSUBAN_KBN.WIKIDIFF, wno, wdd.getSysId());
    wdifd.setItemValue(WikiDiffDto.DIFF_NO, dno);
    wdifd.setItemValue(WikiDiffDto.DIFF_TEXT, txt);
    _wdifa.regist(wdifd);
  }

  /**
   * Describe <code>findDoc</code> method : find WikiDocuments.
   * <br>DB̌B
   *
   * @param wdsd a <code>WikiDocSearchDto</code>
   * @return
   * @throws Exception
   */
  public WikiDocSearchDto findDoc(WikiDocSearchDto wdsd) throws Exception {
    return (WikiDocSearchDto)_wda.find(wdsd);
  }

  /**
   * Describe <code>getDocDetail</code> method : get document inf.
   * <br>̎擾B
   *
   * @param wdda <code>WikiDocDto</code>
   * @return
   * @throws Exception
   */
  public WikiDocDto getDocDetail(WikiDocDto wdd) throws Exception {
    return (WikiDocDto)_wda.findByPrimaryKey(wdd);
  }

  /**
   * Describe <code>getDetail</code> method : get a document.
   * <br>yыL̎擾B
   *
   * @param wdd a <code>WikiDocDto</code>
   * @return a <code>WikiDocDto</code>
   * @throws Exception
   */
  public WikiDocDto getDetailAll(WikiDocDto wdd) throws Exception {
    wdd = getDocDetail(wdd);

    WikiDocument wd = getDocument(wdd);
    wdd.setWikiDoc(wd);
    wdd.setDocument(wd.getOriginal());
    return wdd;
  }

  private WikiDocument getDocument(WikiDocDto wdd) throws Exception {
    WikiDocument wd = new WikiDocument();
    wd.setHeader(wdd.getItemValue(WikiDocDto.TITLE),
                 wdd.getItemValue(WikiDocDto.CATEGORY),
                 wdd.getItemValue(WikiDocDto.SAMMARY));

    String uid = wdd.getItemValue(WikiDocDto.AUTHOR);
    UserDto ud =_scu.getUserInf(uid, wdd.getCommFld(COMMFLD.UUSR));
    String mnm = ud.getItemValue(UserDto.ABBREV);
    if (mnm == null || mnm.length() == 0) mnm = ud.getItemValue(UserDto.NAME);
    wd.setCreate(mnm, wdd.getItemValue(WikiDocDto.CREATE_DATE), wdd.getItemValue(WikiDocDto.CREATE_TIME));

    if (!uid.equals(wdd.getItemValue(WikiDocDto.LAST_AUTHOR))) {
      uid = wdd.getItemValue(WikiDocDto.LAST_AUTHOR);
      ud =_scu.getUserInf(uid, wdd.getCommFld(COMMFLD.UUSR));
      mnm = ud.getItemValue(UserDto.ABBREV);
      if (mnm == null || mnm.length() == 0) mnm = ud.getItemValue(UserDto.NAME);
    }
    wd.setLast(mnm, wdd.getItemValue(WikiDocDto.LAST_DATE), wdd.getItemValue(WikiDocDto.LAST_TIME));
    wd.setDoc(getArticles(wdd));
    return wd;
  }

  private WikiArticle getArticles(WikiDocDto wdd) throws Exception {
    WikiArticleSearchDto wasd = findArticle(wdd);

    WikiArticleDto wad = (WikiArticleDto)CopyDtoUtil.newDto(new DocArctileDtoMapping(), wdd);
    WikiArticle ret = null;
    WikiArticle oya = null;
    String sc = null;
    for (List<String> keys : wasd.getKeys()) {
      wad.setItemValue(WikiArticleDto.WIKI_NO, keys.get(0));
      wad.setItemValue(WikiArticleDto.CHAPTER, keys.get(1));
      wad.setItemValue(WikiArticleDto.SECTION, keys.get(2));
      wad.setAllRequest();
      wad = (WikiArticleDto)_waa.findByPrimaryKey(wad);

      WikiArticle wa = new WikiArticle(
          wad.getItemValue(WikiArticleDto.SUB_TITLE),
          wad.getItemValue(WikiArticleDto.PLAIN_TEXT),
          wad.getItemValue(WikiArticleDto.HTML_TEXT),
          Integer.parseInt(wad.getItemValue(WikiArticleDto.CHAPTER)),
          Integer.parseInt(wad.getItemValue(WikiArticleDto.SECTION)));
      if (ret == null) {
        ret = wa;
        oya = wa;
        sc = keys.get(1);
      } else {
        if (sc.equals(keys.get(1))) {
          wa.setParents(oya);
          oya.addChild(wa);
        } else {
          oya = wa;
          oya.setParents(ret);
          ret.addChild(oya);
          sc = keys.get(1);
        }
      }
    }
    return ret;
  }
  private WikiArticleSearchDto findArticle(WikiDocDto wdd) throws Exception {
    WikiArticleSearchDto wasd = new WikiArticleSearchDto();
    wasd.setSysId(wdd.getSysId());
    wasd.setRetrieveValue(WikiArticleSearchDto.WIKI_NO, wdd.getItemValue(WikiDocDto.WIKI_NO));
    if (!"0".equals(wdd.getChapter())) {
      wasd.setRetrieveValue(WikiArticleSearchDto.CHAPTER, wdd.getChapter());
      if (!"0".equals(wdd.getSection())) {
        wasd.setRetrieveValue(WikiArticleSearchDto.SECTION, wdd.getSection());
      }
    }
    wasd.setOrderItem(WikiArticleSearchDto.CHAPTER, 1);
    wasd.setOrderItem(WikiArticleSearchDto.SECTION, 2);
    wasd = (WikiArticleSearchDto)_waa.find(wasd);
    return wasd;
  }

  /**
   * Describe <code>findDiff</code> method : find diff.
   * <br>ύX̃L[ꗗ̎擾B
   *
   * @param wdsd a <code>WikiDiffSearchDto</code>
   * @return a <code>WikiDiffSearchDto</code>
   * @throws Exception
   */
  public WikiDiffSearchDto findDiff(WikiDiffSearchDto wdsd) throws Exception {
    return (WikiDiffSearchDto)_wdifa.find(wdsd);
  }

  /**
   * Describe <code>getDiff</code> method : get diff detail.
   * <br>ύX擾B
   *
   * @param wdd a <code>WikiDiffDto</code>
   * @return
   * @throws Exception
   */
  public WikiDiffDto getDiff(WikiDiffDto wdd) throws Exception {
    return (WikiDiffDto)_wdifa.findByPrimaryKey(wdd);
  }

  /**
   * Describe <code>isExistDiff</code> method : .
   * <br>Y̕ύXg̑݃`FbNsB
   * <br>݂鎞Atrue߂B
   *
   * @param wno a <Code>String</code> : ԍ
   * @param sysId a <code>String</code> : system Id
   * @return a <code>boolean</code> : ݂Ƃ ^
   * @throws Exception
   */
  public boolean isExistDiff(String wno, String sysId) throws Exception {
    WikiDiffSearchDto wdsd = new WikiDiffSearchDto();
    wdsd.setSysId(sysId);
    wdsd.setRetrieveValue(WikiDiffSearchDto.WIKI_NO, wno);
    wdsd = (WikiDiffSearchDto)_wdifa.find(wdsd);
    if (wdsd.getKeys().size() == 0) return false;
    return true;
  }
}
