package com.hamaf.business.workflow.process;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.hamaf.business.workflow.access.BaseAuthorizerAccess;
import com.hamaf.business.workflow.access.BaseFlowAccess;
import com.hamaf.business.workflow.access.BaseFlowAccess.USER_KUBUN;
import com.hamaf.business.workflow.access.BaseLevelAccess;
import com.hamaf.business.workflow.access.BaseLevelAccess.AUTHORIZER_KUBUN;
import com.hamaf.business.workflow.access.HatsubanAccess;
import com.hamaf.business.workflow.access.HatsubanAccess.HATSUBAN_KBN;
import com.hamaf.business.workflow.access.SqlExec;
import com.hamaf.business.workflow.dto.BaseAuthorizerDto;
import com.hamaf.business.workflow.dto.BaseAuthorizerSearchDto;
import com.hamaf.business.workflow.dto.BaseFlowDto;
import com.hamaf.business.workflow.dto.BaseFlowSearchDto;
import com.hamaf.business.workflow.dto.BaseLevelDto;
import com.hamaf.business.workflow.dto.BaseLevelSearchDto;
import com.hamaf.business.workflow.dto.BaseFlowLevelDtoMapping;
import com.hamaf.business.workflow.dto.BaseLevelAuthorizeDtoMapping;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface.COMMFLD;
import com.hamaf.hikyaku.dataobject.DetailItemsInterface.USE_FLG;
import com.hamaf.hikyaku.dataobject.PreparedItems;
import com.hamaf.hikyaku.dataobject.utils.CopyDtoUtil;
import com.hamaf.hikyaku.security.SecurityCheckUtils;
import com.hamaf.hikyaku.security.dto.GroupDto;
import com.hamaf.hikyaku.security.dto.UserDto;

/**
 * <p>^CgFBaseFlowProcess is .</p>
 * <p>F</p>
 * <br>
 *
 * <p>쌠: Copyright (c) 2012/10/05</p>
 * @author Yoshinori
 * @version 1.0
 *
 */
public class BaseFlowProcess {

  private BaseFlowAccess _bfa = null;
  private BaseLevelAccess _bla = null;
  private BaseAuthorizerAccess _baa = null;
  private HatsubanAccess _ha = null;
  private SqlExec _se = null;
  private SecurityCheckUtils _scu = null;
  
  /**
   * Creates a new <code>BaseFlowProcess</code> instance.
   * <br>Default Constructor
   */
  public BaseFlowProcess() {
    _baa = new BaseAuthorizerAccess();
    _bla = new BaseLevelAccess();
    _bfa = new BaseFlowAccess();
    _ha = new HatsubanAccess();
    _se = new SqlExec(); 
    _scu = new SecurityCheckUtils();
  }

  public BaseFlowSearchDto findFlow(BaseFlowSearchDto bfsd) throws Exception {
    return (BaseFlowSearchDto)_bfa.find(bfsd);
  }
  
  public BaseFlowDto getFlowDetail(BaseFlowDto bfd) throws Exception {
    return (BaseFlowDto)_bfa.findByPrimaryKey(bfd);
  }
  
  public BaseFlowDto getFlowDetail(String fno, String sysid) throws Exception {
    BaseFlowDto bfd = new BaseFlowDto();
    bfd.setItemValue(BaseFlowDto.FLOW_NO, fno);
    bfd.setSysId(sysid);
    bfd.setAllRequest();
    return getFlowDetail(bfd);
  }
  
  public BaseLevelSearchDto findLevel(BaseLevelSearchDto blsd) throws Exception {
    return (BaseLevelSearchDto)_bla.find(blsd);
  }
  
  public BaseLevelDto getLeveDetail(BaseLevelDto bld) throws Exception {
    bld = (BaseLevelDto)_bla.findByPrimaryKey(bld);
    
    String [] val = findAuthorizer(bld, USER_KUBUN.USER);
    bld.setUsers(val[0]);
    bld.setUnames(val[1]);
    val = findAuthorizer(bld, USER_KUBUN.GROUP);
    bld.setGroups(val[0]);
    bld.setGnames(val[1]);

    return bld;
  }
  private String[] findAuthorizer(BaseLevelDto bld, USER_KUBUN uk) throws Exception {
    BaseAuthorizerSearchDto basd = new BaseAuthorizerSearchDto();
    basd.setSysId(bld.getSysId());
    basd.setRetrieveValue(BaseAuthorizerSearchDto.FLOW_NO, bld.getItemValue(BaseLevelDto.FLOW_NO));
    basd.setRetrieveValue(BaseAuthorizerSearchDto.LEVEL, bld.getItemValue(BaseLevelDto.LEVEL));
    basd.setRetrieveValue(BaseAuthorizerSearchDto.AUTHORIZER_KUBUN, bld.getItemValue(BaseLevelDto.AUTHORIZER_KUBUN));
    basd.setRetrieveValue(BaseAuthorizerSearchDto.USER_KUBUN, uk.getVal());
    basd.setOrderItem(BaseAuthorizerSearchDto.USER_ID, 1);
    basd = (BaseAuthorizerSearchDto)_baa.find(basd);

    StringBuilder sb = new StringBuilder();
    StringBuilder sbn = new StringBuilder();
    for (List<String> keys : basd.getKeys()) {
      sb.append(keys.get(4));
      sb.append(",");
      
      sbn.append(keys.get(4));
      sbn.append("-");
      if (uk.equals(USER_KUBUN.USER)) {
        UserDto ud = _scu.getUserInf(keys.get(4), bld.getCommFld(COMMFLD.UUSR));
        sbn.append(ud.getItemValue(UserDto.NAME));
      } else {
        GroupDto gd = _scu.getGroupInf(keys.get(4), bld.getCommFld(COMMFLD.UUSR));
        sbn.append(gd.getItemValue(GroupDto.NAME));
      }
      sbn.append(",");
    }
    if (sb.length() > 0) {
      sb.setLength(sb.length() - 1);
      sbn.setLength(sbn.length() - 1);
    }
    String[] ret = {sb.toString(), sbn.toString()};
    return ret;
  }
  
  public String addNewFlow(BaseFlowDto bfd, List<BaseLevelDto> lvl) throws Exception {
    String no = _ha.getNextNo(HATSUBAN_KBN.FLOW, null, bfd.getSysId());
    bfd.setItemValue(BaseFlowDto.FLOW_NO, no);
    bfd.setItemValue(BaseFlowDto.LEVEL_COUNT, checkLebel(lvl));
    _bfa.regist(bfd);

    addLebels(bfd, lvl);
    return no;
  }
  private String checkLebel(List<BaseLevelDto> lvl) {
    int ilv = 0;
    for (BaseLevelDto bld : lvl) {
      if (AUTHORIZER_KUBUN.AUTHORIZE.getVal().equals(bld.getItemValue(BaseLevelDto.AUTHORIZER_KUBUN))) {
        ilv++;
      }
    }
    return String.valueOf(ilv);
  }
  private void addLebels(BaseFlowDto bfd, List<BaseLevelDto> lvl) throws Exception {
    for (BaseLevelDto olddto : lvl) {
      BaseLevelDto bld = (BaseLevelDto)CopyDtoUtil.newDto(new BaseFlowLevelDtoMapping(), bfd);
      for (int i = 1; i < olddto.getItemCount(); i++) {
        bld.setItemValue(i, olddto.getItemValue(i));
      }
      _bla.regist(bld);
      
      BaseAuthorizerDto bad = (BaseAuthorizerDto)CopyDtoUtil.newDto(new BaseLevelAuthorizeDtoMapping(), bld);
      String grps = olddto.getGroups();
      if (grps != null && grps.length() > 0) {
        String[] grp = grps.split(",");
        for (int j = 0; j < grp.length; j++) {
          bad.setItemValue(BaseAuthorizerDto.USER_KUBUN, USER_KUBUN.GROUP.getVal());
          bad.setItemValue(BaseAuthorizerDto.USER_ID, grp[j]);
          _baa.regist(bad);
        }
      }
      String users = olddto.getUsers();
      if (users != null && users.length() > 0) {
        String[] usr = users.split(",");
        for (int j = 0; j < usr.length; j++) {
          bad.setItemValue(BaseAuthorizerDto.USER_KUBUN, USER_KUBUN.USER.getVal());
          bad.setItemValue(BaseAuthorizerDto.USER_ID, usr[j]);
          _baa.regist(bad);
        }
      }
    }
  }
  
  public void updateFlow(BaseFlowDto bfd, List<BaseLevelDto> lvl) throws Exception {
    bfd.setItemValue(BaseFlowDto.LEVEL_COUNT, checkLebel(lvl));
    _bfa.update(bfd);
    
    deleteLevel(bfd);
    addLebels(bfd, lvl);
  }
  private void deleteLevel(BaseFlowDto bfd) throws Exception {
    PreparedItems pi = new PreparedItems();
    pi.setSysId(bfd.getSysId());
    Map<String, Object> prm = new HashMap<String, Object>();
    prm.put("FLOW_NO", bfd.getItemValue(BaseFlowDto.FLOW_NO));
    pi.addParam(prm);
    _se.delBaseLevel(pi);
    _se.delBaseAuthorizer(pi);
  }
  
  public void deleteFlow(BaseFlowDto bfd) throws Exception {
    bfd.setCommFld(COMMFLD.DEL, USE_FLG.DEL.getVal());
    _bfa.update(bfd);
  }
}
